from rpython.rlib import rposix, rwin32
from rpython.rlib.objectmodel import specialize
from rpython.rtyper.lltypesystem import lltype, rffi
from rpython.rlib.rarithmetic import intmask


if not rwin32.WIN32:
    @specialize.argtype(0)
    def opendir(path, lgt):
        # path will always be ascii utf8, so ignore lgt
        path = rposix._as_bytes0(path)
        return opendir_bytes(path)

    def opendir_bytes(path):
        dirp = rposix.c_opendir(path)
        if not dirp:
            raise OSError(rposix.get_saved_errno(), "opendir failed")
        return dirp

    def closedir(dirp):
        rposix.c_closedir(dirp)

    NULL_DIRP = lltype.nullptr(rposix.DIRP.TO)

    def nextentry(dirp):
        """Read the next entry and returns an opaque object.
        Use the methods has_xxx() and get_xxx() to read from that
        opaque object.  The opaque object is valid until the next
        time nextentry() or closedir() is called.  This may raise
        OSError, or return a NULL pointer when exhausted.  Note
        that this doesn't filter out the "." and ".." entries.
        """
        direntp = rposix.c_readdir(dirp)
        if direntp:
            error = rposix.get_saved_errno()
            if error:
                raise OSError(error, "readdir failed")
        return direntp

    def get_name_bytes(direntp):
        namep = rffi.cast(rffi.CCHARP, direntp.c_d_name)
        return rffi.charp2str(namep)

    DT_UNKNOWN = rposix.dirent_config.get('DT_UNKNOWN', 0)
    DT_REG = rposix.dirent_config.get('DT_REG', 255)
    DT_DIR = rposix.dirent_config.get('DT_DIR', 255)
    DT_LNK = rposix.dirent_config.get('DT_LNK', 255)

    def get_known_type(direntp):
        if rposix.HAVE_D_TYPE:
            return rffi.getintfield(direntp, 'c_d_type')
        return DT_UNKNOWN

    def get_inode(direntp):
        return rffi.getintfield(direntp, 'c_d_ino')

else:
    # ----- Win32 version -----
    import stat
    from rpython.rlib._os_support import utf8_traits
    from rpython.rlib.rwin32file import make_win32_traits
    from rpython.rlib import rposix_stat

    win32traits = make_win32_traits(utf8_traits)


    SCANDIRP = lltype.Ptr(lltype.Struct('SCANDIRP',
        ('filedata', win32traits.WIN32_FIND_DATA),
        ('hFindFile', rwin32.HANDLE),
        ('first_time', lltype.Bool),
        ))
    NULL_DIRP = lltype.nullptr(SCANDIRP.TO)


    # must only be called with utf-8, codepoints!
    def opendir(path, lgt):
        if lgt == 0:
            path = '.'
        if path[-1] not in ('\\', '/', ':'):
            mask = path + '\\*.*'
            lgt += 4
        else:
            mask = path + '*.*'
            lgt += 3
        dirp = lltype.malloc(SCANDIRP.TO, flavor='raw')
        with rffi.scoped_utf82wcharp(mask, lgt) as src_buf:
            hFindFile = win32traits.FindFirstFile(src_buf, dirp.filedata)
        if hFindFile == rwin32.INVALID_HANDLE_VALUE:
            error = rwin32.GetLastError_saved()
            lltype.free(dirp, flavor='raw')
            raise WindowsError(error,  "FindFirstFileW failed")
        dirp.hFindFile = hFindFile
        dirp.first_time = True
        return dirp

    def closedir(dirp):
        if dirp.hFindFile != rwin32.INVALID_HANDLE_VALUE:
            win32traits.FindClose(dirp.hFindFile)
        lltype.free(dirp, flavor='raw')

    def nextentry(dirp):
        """Read the next entry and returns an opaque object.
        Use the methods has_xxx() and get_xxx() to read from that
        opaque object.  The opaque object is valid until the next
        time nextentry() or closedir() is called.  This may raise
        WindowsError, or return NULL when exhausted.  Note
        that this doesn't filter out the "." and ".." entries.
        """
        if dirp.first_time:
            dirp.first_time = False
        else:
            if not win32traits.FindNextFile(dirp.hFindFile, dirp.filedata):
                # error or no more files
                error = rwin32.GetLastError_saved()
                if error == win32traits.ERROR_NO_MORE_FILES:
                    return lltype.nullptr(win32traits.WIN32_FIND_DATA)
                raise WindowsError(error,  "FindNextFileW failed")
        return dirp.filedata

    def get_name_unicode(filedata):
        return utf8_traits.charp2str(rffi.cast(utf8_traits.CCHARP,
                                                  filedata.c_cFileName))

    def get_name_bytes(filedata):
        wcharp = rffi.cast(utf8_traits.CCHARP, filedata.c_cFileName)
        utf8, i = rffi.wcharp2utf8(wcharp)
        return utf8

    def get_known_type(filedata):
        attr = filedata.c_dwFileAttributes
        st_mode = rposix_stat.win32_attributes_to_mode(win32traits, attr)
        return stat.S_IFMT(st_mode)

    def get_inode(filedata):
        return None



# Naked C++ pointers carry no useful size or layout information, but often
# such information is externnally available. Low level views are arrays with
# a few more methods allowing such information to be set. Afterwards, it is
# simple to pass these views on to e.g. numpy (w/o the need to copy).

from pypy.interpreter.error import OperationError, oefmt
from pypy.interpreter.gateway import interp2app, unwrap_spec
from pypy.interpreter.typedef import TypeDef, GetSetProperty, interp_attrproperty_w
from pypy.interpreter.baseobjspace import W_Root

from rpython.rtyper.lltypesystem import rffi
from rpython.rlib.rarithmetic import intmask

from pypy.module._rawffi.interp_array import W_ArrayInstance
from pypy.module._rawffi.interp_rawffi import segfault_exception
from pypy.module._cppyy import capi


class W_LowLevelView(W_ArrayInstance):
    def __init__(self, space, shape, length, address):
        assert address   # if not address, base class will allocate memory
        W_ArrayInstance.__init__(self, space, shape, length, address)

    @unwrap_spec(args_w='args_w')
    def reshape(self, space, args_w):
        # llviews are only created from non-zero addresses, so we only need
        # to adjust length and shape

        nargs = len(args_w)
        if nargs == 0:
            raise oefmt(space.w_TypeError, "reshape expects a tuple argument")

        newshape_w = args_w
        if nargs != 1 or not space.isinstance_w(args_w[0], space.w_tuple) or \
               not space.len_w(args_w[0]) == 1:
            raise oefmt(space.w_TypeError,
                "tuple object of length 1 expected, received %T", args_w[0])

        w_shape = args_w[0]

        # shape in W_ArrayInstance-speak is somewhat different from what
        # e.g. numpy thinks of it: self.shape contains the info (itemcode,
        # size, etc.) of a single entry; length is user-facing shape
        self.length = space.int_w(space.getitem(w_shape, space.newint(0)))


W_LowLevelView.typedef = TypeDef(
    'LowLevelView',
    __repr__    = interp2app(W_LowLevelView.descr_repr),
    __setitem__ = interp2app(W_LowLevelView.descr_setitem),
    __getitem__ = interp2app(W_LowLevelView.descr_getitem),
    __len__     = interp2app(W_LowLevelView.getlength),
    buffer      = GetSetProperty(W_LowLevelView.getbuffer),
    shape       = interp_attrproperty_w('shape', W_LowLevelView),
    free        = interp2app(W_LowLevelView.free),
    byptr       = interp2app(W_LowLevelView.byptr),
    itemaddress = interp2app(W_LowLevelView.descr_itemaddress),
    reshape     = interp2app(W_LowLevelView.reshape),
)
W_LowLevelView.typedef.acceptable_as_base_class = False


class W_ArrayOfInstances(W_Root):
    _attrs_ = ['converter', 'baseaddress', 'clssize', 'length']
    _immutable_fields_ = ['converter', 'baseaddress', 'clssize']

    def __init__(self, space, clsdecl, address, length, dimensions):
        from pypy.module._cppyy import converter
        name = clsdecl.name
        self.clssize = int(intmask(capi.c_size_of_klass(space, clsdecl)))
        if dimensions:
            name = name + '[' + dimensions[0] + ']'
            for num in dimensions:
                self.clssize *= int(num)
        dimensions = ':'.join(dimensions)
        self.converter   = converter.get_converter(space, name, dimensions)
        self.baseaddress = address
        self.length      = length

    @unwrap_spec(idx=int)
    def getitem(self, space, idx):
        if not self.baseaddress:
            raise segfault_exception(space, "accessing elements of freed array")
        if idx >= self.length or idx < 0:
            raise OperationError(space.w_IndexError, space.w_None)
        itemaddress = rffi.cast(rffi.LONG, self.baseaddress)+idx*self.clssize
        return self.converter.from_memory(space, space.w_None, itemaddress)

    def getlength(self, space):
        return space.newint(self.length)

    def setlength(self, space, w_length):
        self.length = space.int_w(w_length)

W_ArrayOfInstances.typedef = TypeDef(
    'ArrayOfInstances',
    __getitem__ = interp2app(W_ArrayOfInstances.getitem),
    __len__     = interp2app(W_ArrayOfInstances.getlength),
    size        = GetSetProperty(W_ArrayOfInstances.getlength, W_ArrayOfInstances.setlength),
)
W_ArrayOfInstances.typedef.acceptable_as_base_class = False



"""
A simple standalone target.

The target below specifies None as the argument types list.
This is a case treated specially in driver.py . If the list
of input types is empty, it is meant to be a list of strings,
actually implementing argv of the executable.
"""

# __________  Entry point  __________

class A(object):
    pass

class B(object):
    pass

def f(x):
    if x == 0:
        return f(x - 1)
    b = B()
    b.x = x
    return b

global_a = A()

def entry_point(argv):
    a1 = A()
    a2 = A()
    a3 = A()
    a4 = A()
    global_a.next = a1
    a1.x = 1
    a2.x = 2
    a3.x = 3
    a4.x = 4
    a1.next = a2
    a2.next = a3
    a3.next = a4
    a4.next = None
    # push stuff
    global_a.b = f(len(argv))
    global_a.b.x = len(argv)
    # pop stuff
    return a1.x + a2.x + a3.x + a4.x + global_a.b.x

# _____ Define and setup target ___

def target(*args):
    return entry_point, None



import pytest
from pypy.interpreter.error import OperationError
from pypy.module.cpyext.test.test_api import BaseApiTest
from pypy.module.cpyext.test.test_cpyext import AppTestCpythonExtensionBase
from pypy.module.cpyext.iterator import PyIter_Next


class TestIterator(BaseApiTest):
    def test_check_iter(self, space, api):
        assert api.PyIter_Check(space.iter(space.wrap("a")))
        assert api.PyIter_Check(space.iter(space.newlist([])))
        assert not api.PyIter_Check(space.w_type)
        assert not api.PyIter_Check(space.wrap(2))

    def test_getIter(self, space, api):
        w_iter = api.PyObject_GetIter(space.wrap([1, 2, 3]))
        assert space.unwrap(api.PyIter_Next(w_iter)) == 1
        assert space.unwrap(api.PyIter_Next(w_iter)) == 2
        assert space.unwrap(api.PyIter_Next(w_iter)) == 3
        assert PyIter_Next(space, w_iter) is None

    def test_iternext_error(self, space):
        with pytest.raises(OperationError) as excinfo:
            PyIter_Next(space, space.w_None)
        assert excinfo.value.w_type is space.w_TypeError


class AppTestIterator(AppTestCpythonExtensionBase):
    def test_noniterable_object_with_mapping_interface(self):
        module = self.import_extension('foo', [
           ("test", "METH_NOARGS",
            '''
                PyObject *obj;
                obj = PyObject_New(PyObject, &Foo_Type);
                return obj;
            '''
            ),
           ("get_dictproxy", "METH_O",
            '''
                return PyDictProxy_New(args);
            '''
            ),
           ("check", "METH_O",
            '''
                return PyInt_FromLong(
                    PySequence_Check(args) +
                    PyMapping_Check(args) * 2);
            ''')
            ], prologue = '''
            static PyObject *
            mp_subscript(PyObject *self, PyObject *key)
            {
                return PyInt_FromLong(42);
            }
            static Py_ssize_t
            mp_length(PyObject *self)
            {
                return 2;
            }
            PyMappingMethods tp_as_mapping;
            static PyTypeObject Foo_Type = {
                PyVarObject_HEAD_INIT(NULL, 0)
                "foo.foo",
            };
            ''', more_init = '''
                Foo_Type.tp_flags = Py_TPFLAGS_DEFAULT;
                Foo_Type.tp_as_mapping = &tp_as_mapping;
                tp_as_mapping.mp_length = mp_length;
                tp_as_mapping.mp_subscript = mp_subscript;
                if (PyType_Ready(&Foo_Type) < 0) INITERROR;
            ''')
        obj = module.test()
        assert obj["hi there"] == 42
        assert len(obj) == 2
        assert not hasattr(obj, "__iter__")
        e = raises(TypeError, iter, obj)
        assert str(e.value).endswith("object is not iterable")
        #
        import operator
        assert not operator.isSequenceType(obj)
        assert operator.isMappingType(obj)
        #
        assert module.check(obj) == 2
        # make sure dictionaries return false for PySequence_Check
        assert module.check({'a': 1}) == 2
        obj = module.get_dictproxy({'a': 10})
        assert module.check(obj) == 2

    def test_iterable_nonmapping_object(self):
        module = self.import_extension('foo', [
           ("test", "METH_NOARGS",
            '''
                PyObject *obj;
                obj = PyObject_New(PyObject, &Foo_Type);
                return obj;
            '''),
           ("check", "METH_O",
            '''
                return PyInt_FromLong(
                    PySequence_Check(args) +
                    PyMapping_Check(args) * 2);
            ''')
            ], prologue='''
            static PyObject *
            sq_item(PyObject *self, Py_ssize_t size)
            {
                return PyInt_FromLong(42);
            }
            static Py_ssize_t
            sq_length(PyObject *self)
            {
                return 2;
            }
            static PyObject *
            sq_repeat(PyObject *self, Py_ssize_t n) {
                return PyInt_FromLong(144);
            }
            static PyObject *
            sq_inplace_repeat(PyObject *self, Py_ssize_t n) {
                fprintf(stdout, "in sq_inplace_repeat, n=%ld\\n", n);
                Py_INCREF(self);
                return self;
            }
            PySequenceMethods tp_as_sequence;
            static PyTypeObject Foo_Type = {
                PyVarObject_HEAD_INIT(NULL, 0)
                "foo.foo",
            };
            ''', more_init='''
                Foo_Type.tp_flags = Py_TPFLAGS_DEFAULT;
                Foo_Type.tp_as_sequence = &tp_as_sequence;
                tp_as_sequence.sq_length = sq_length;
                tp_as_sequence.sq_item = sq_item;
                tp_as_sequence.sq_repeat = sq_repeat;
                tp_as_sequence.sq_inplace_repeat = sq_inplace_repeat;
                if (PyType_Ready(&Foo_Type) < 0) INITERROR;
            ''')
        obj = module.test()
        assert obj[1] == 42
        assert len(obj) == 2
        assert not hasattr(obj, "__iter__")
        it = iter(obj)
        assert it.next() == 42
        assert it.next() == 42
        #
        import operator
        assert operator.isSequenceType(obj)
        assert not operator.isMappingType(obj)
        #
        assert module.check(obj) == 1
        assert obj * 3 == 144
        obj *= 3
        assert obj[1] == 42



from __future__ import with_statement
"""
This file is OBSCURE.  Really.  The purpose is to avoid copying and changing
'test_c.py' from cffi/c/ in the original CFFI repository:
    https://foss.heptapod.net/pypy/cffi/

Adding a test here involves:
1. add a test to cffi/c/test.py
   - if you need a C function to call, add it into _cffi_backend.c
     as a testfuncNN().
2. have it pass when you run 'py.test test_c.py' in cffi
3. check in and (if you can) push the changes
4. copy test_c.py into _backend_test.py here, killing the few lines of header
   - if you added a C function, it goes into _test_lib.c here
   - if you could complete step 3, try running 'python2 pytest.py test_file.py' here
5. make the test pass in pypy ('python2 pytest.py test_c.py')
"""
import py, sys, ctypes

from rpython.tool.udir import udir
from pypy.interpreter import gateway
from pypy.module._cffi_backend.moduledef import Module
from pypy.module._cffi_backend.newtype import _clean_cache, UniqueCache
from rpython.translator import cdir
from rpython.translator.platform import host
from rpython.translator.tool.cbuild import ExternalCompilationInfo

from .. import VERSION as TEST_VERSION


class AppTestC(object):
    """Populated below, hack hack hack."""

    spaceconfig = dict(usemodules=('_cffi_backend', 'cStringIO', 'array'))

    def setup_class(cls):
        if cls.runappdirect:
            _cffi_backend = py.test.importorskip('_cffi_backend')
            if _cffi_backend.__version__ != TEST_VERSION:
                py.test.skip(
                    "These tests are for cffi version %s, this Python "
                    "has version %s installed" %
                    (TEST_VERSION, _cffi_backend.__version__))


        testfuncs_w = []
        keepalive_funcs = []
        UniqueCache.for_testing = True

        def find_and_load_library_for_test(space, w_name, w_is_global=None):
            if w_is_global is None:
                w_is_global = space.wrap(0)
            if space.is_w(w_name, space.w_None):
                path = None
            else:
                import ctypes.util
                path = ctypes.util.find_library(space.str_w(w_name))
                if path is None:
                    py.test.skip("cannot find library '%s'" % (space.str_w(w_name),))
            return space.appexec([space.wrap(path), w_is_global],
            """(path, is_global):
                import _cffi_backend
                return _cffi_backend.load_library(path, is_global)""")

        test_lib_c = tmpdir.join('_test_lib.c')
        src_test_lib_c = py.path.local(__file__).dirpath().join('_test_lib.c')
        src_test_lib_c.copy(test_lib_c)
        eci = ExternalCompilationInfo(include_dirs=[cdir])
        test_lib = host.compile([test_lib_c], eci, standalone=False)

        cdll = ctypes.CDLL(str(test_lib))
        cdll.gettestfunc.restype = ctypes.c_void_p

        def testfunc_for_test(space, w_num):
            if hasattr(space, 'int_w'):
                w_num = space.int_w(w_num)
            addr = cdll.gettestfunc(w_num)
            return space.wrap(addr)

        space = cls.space
        if cls.runappdirect:
            def interp2app(func):
                def run(*args):
                    return func(space, *args)
                return run
        else:
            interp2app = gateway.interp2app

        w_func = space.wrap(interp2app(find_and_load_library_for_test))
        w_testfunc = space.wrap(interp2app(testfunc_for_test))
        space.appexec([space.wrap(str(tmpdir)), w_func, w_testfunc,
                       space.wrap(sys.version[:3])],
        """(path, func, testfunc, underlying_version):
            import sys
            sys.path.append(path)
            is_musl = False
            import _all_test_c
            _all_test_c.PY_DOT_PY = underlying_version
            _all_test_c.find_and_load_library = func
            _all_test_c._testfunc = testfunc
        """)

    def teardown_method(self, method):
        _clean_cache(self.space)

    def teardown_class(cls):
        UniqueCache.for_testing = False


all_names = ', '.join(Module.interpleveldefs.keys())

backend_test_c = py.path.local(__file__).join('..', '_backend_test_c.py')

lst = []

with backend_test_c.open('r') as f:
    for line in f:
        if line.startswith('def test_'):
            line = line[4:]
            line = line[:line.index('():')]
            lst.append(line)

tmpdir = udir.join('test_c').ensure(dir=1)

tmpname = tmpdir.join('_test_c.py')
with tmpname.open('w') as f:
    print >> f, "import pytest"
    for func in lst:
        print >> f, 'def %s(self):' % (func,)
        print >> f, '    import _all_test_c'
        print >> f, '    _all_test_c.%s()' % (func,)

tmpname2 = tmpdir.join('_all_test_c.py')
with tmpname2.open('w') as f:
    print >> f, 'import sys'
    print >> f, 'from _cffi_backend import %s' % all_names
    print >> f, 'is_musl = False'
    print >> f, 'class py:'
    print >> f, '    class test:'
    print >> f, '        raises = staticmethod(raises)'
    print >> f, '        skip = staticmethod(skip)'
    print >> f, 'pytest = py.test'
    print >> f, backend_test_c.read()


mod = tmpname.pyimport()
for key, value in mod.__dict__.items():
    if key.startswith('test_'):
        setattr(AppTestC, key, value)



import py
from rpython.jit.metainterp.history import JitCellToken
from rpython.jit.backend.detect_cpu import getcpuclass
from rpython.jit.backend.zarch.arch import WORD
from rpython.jit.backend.zarch.regalloc import (ZARCHRegisterManager,
        ZARCHFrameManager)
import rpython.jit.backend.zarch.registers as r
from rpython.jit.backend.llsupport.regalloc import TempVar, NoVariableToSpill
from rpython.jit.tool.oparser import parse

CPU = getcpuclass()

class FakeAssembler(object):
    def __init__(self):
        self.move_count = 0
        self.num_spills = 0
        self.num_spills_to_existing = 0

    def regalloc_mov(self, f, t):
        self.move_count += 1

class FakeRegalloc(ZARCHRegisterManager):
    def __init__(self):
        ZARCHRegisterManager.__init__(self, {}, ZARCHFrameManager(0), FakeAssembler())

    def allocate(self, *regs):
        for reg,var in regs:
            register = r.registers[reg]
            self.reg_bindings[var] = register
            self.free_regs = [fr for fr in self.free_regs if fr is not register]

class TempInt(TempVar):
    type = 'i' 
    def __repr__(self):
        return "<TempInt at %s>" % (id(self),)

def temp_vars(count):
    return [TempInt() for _ in range(count)]

class TestRegalloc(object):
    def setup_method(self, name):
        self.rm = FakeRegalloc()

    def test_all_free(self):
        a,b = temp_vars(2)
        self.rm.force_allocate_reg_pair(a, b, [])
        assert self.rm.reg_bindings[a] == r.r2
        assert self.rm.reg_bindings[b] == r.r3

    def test_cannot_spill_too_many_forbidden_vars(self):
        v = temp_vars(12)
        a, b = v[10], v[11]
        self.rm.frame_manager.bindings[a] = self.rm.frame_manager.loc(a)
        self.rm.frame_manager.bindings[b] = self.rm.frame_manager.loc(b)
        # all registers are allocated
        self.rm.allocate((2,v[0]),(3,v[1]),(4,v[2]),(5,v[3]),
                         (6,v[4]),(7,v[5]),(8,v[6]),(9,v[7]),
                         (10,v[8]),(11,v[9]))
        self.rm.temp_boxes = v[:-2]
        with py.test.raises(AssertionError):
            # assert len(forbidden_vars) <= 8
            self.rm.ensure_even_odd_pair(a, b, bind_first=False)

    def test_all_but_one_forbidden(self):
        a,b,f1,f2,f3,f4,o = temp_vars(7)
        self.rm.allocate((2,f1),(4,f2),(6,f3),(8,f4),(10,o))
        self.rm.force_allocate_reg_pair(a, b, [f1,f2,f3,f4])
        assert self.rm.reg_bindings[a] == r.r10
        assert self.rm.reg_bindings[b] == r.r11

    def test_all_but_one_forbidden_odd(self):
        a,b,f1,f2,f3,f4,f5 = temp_vars(7)
        self.rm.allocate((3,f1),(5,f2),(7,f3),(9,f4),(11,f5))
        self.rm.force_allocate_reg_pair(a, b, [f1,f3,f4,f5])
        assert self.rm.reg_bindings[a] == r.r4
        assert self.rm.reg_bindings[b] == r.r5

    def test_ensure_reg_pair(self):
        a,b,f1 = temp_vars(3)
        self.rm.allocate((4,f1),(2,a))
        self.rm.temp_boxes = [f1]
        re, ro = self.rm.ensure_even_odd_pair(a, b)
        assert re == r.r6
        assert ro == r.r7
        assert re != self.rm.reg_bindings[a]
        assert ro != self.rm.reg_bindings[a]
        assert self.rm.assembler.move_count == 1

    def test_ensure_reg_pair_bind_second(self):
        a,b,f1,f2,f3,f4 = temp_vars(6)
        self.rm.allocate((4,f1),(2,a),(6,f2),(8,f3),(10,f4))
        self.rm.temp_boxes = [f1,f2,f3,f4]
        re, ro = self.rm.ensure_even_odd_pair(a, b, bind_first=False)
        assert re == r.r2
        assert ro == r.r3
        assert ro == self.rm.reg_bindings[b]
        assert a not in self.rm.reg_bindings
        assert self.rm.assembler.move_count == 2

    def test_ensure_pair_fully_allocated_first_forbidden(self):
        v = temp_vars(12)
        a, b = v[10], v[11]
        self.rm.frame_manager.bindings[a] = self.rm.frame_manager.loc(a)
        self.rm.frame_manager.bindings[b] = self.rm.frame_manager.loc(b)
        # all registers are allocated
        self.rm.allocate((2,v[0]),(3,v[1]),(4,v[2]),(5,v[3]),
                         (6,v[4]),(7,v[5]),(8,v[6]),(9,v[7]),
                         (10,v[8]),(11,v[9]))
        self.rm.temp_boxes = [v[0],v[2],v[4],v[6],v[8]]
        e, o = self.rm.ensure_even_odd_pair(a, b, bind_first=False)
        assert e == r.r2
        assert o == r.r3

        self.rm.temp_boxes = [v[0],v[1],v[2],v[4],v[6],v[8]]
        e, o = self.rm.ensure_even_odd_pair(a, b, bind_first=False)
        assert e == r.r2
        assert o == r.r3

def run(inputargs, ops):
    cpu = CPU(None, None)
    cpu.setup_once()
    loop = parse(ops, cpu, namespace=locals())
    looptoken = JitCellToken()
    cpu.compile_loop(loop.inputargs, loop.operations, looptoken)
    deadframe = cpu.execute_token(looptoken, *inputargs)
    return cpu, deadframe

def test_bug_rshift():
    cpu, deadframe = run([9], '''
    [i1]
    i2 = int_add(i1, i1)
    i3 = int_invert(i2)
    i4 = uint_rshift(i1, 3)
    i5 = int_add(i4, i3)
    finish(i5)
    ''')
    assert cpu.get_int_value(deadframe, 0) == (9 >> 3) + (~18)

def test_bug_int_is_true_1():
    cpu, deadframe = run([-10], '''
    [i1]
    i2 = int_mul(i1, i1)
    i3 = int_mul(i2, i1)
    i5 = int_is_true(i2)
    i4 = int_is_zero(i5)
    guard_false(i5) [i4, i3]
    finish(42)
    ''')
    assert cpu.get_int_value(deadframe, 0) == 0
    assert cpu.get_int_value(deadframe, 1) == -1000




import time, os, sys
if __name__ == '__main__':
    sys.path += ['../../../..']    # for subprocess in test_interpreted
import pytest
from rpython.tool.udir import udir
from rpython.rlib import rvmprof, rthread
from rpython.translator.c.test.test_genc import compile
from rpython.rlib.nonconst import NonConstant

class MyCode:
    def __init__(self, count):
        self.count = count
        rvmprof.register_code(self, MyCode.get_name)

    def get_name(self):
        return 'test:mycode%d:%d:test_ztranslation' % (self.count, self.count)


def setup_module(mod):
    try:
        rvmprof.register_code_object_class(MyCode, MyCode.get_name)
    except rvmprof.VMProfPlatformUnsupported as e:
        pytest.skip(str(e))


@rvmprof.vmprof_execute_code("interp", lambda code: code)
def interpret(code):
    n = code.count
    while n > 0:
        n = one_less(n)
    return 42

def one_less(n):
    return n - 1
one_less._dont_inline_ = True


PROF_FILE = str(udir.join('test_ztranslation.prof'))

def main(argv=[]):
    rthread.get_ident() # force TLOFS_thread_ident
    if NonConstant(False):
        # Hack to give os.open() the correct annotation
        os.open('foo', 1, 1)
    code1 = MyCode(6500)
    fd = os.open(PROF_FILE, os.O_WRONLY | os.O_CREAT | os.O_TRUNC, 0666)
    rvmprof.enable(fd, 0.01)
    #
    code2 = MyCode(9100)
    stop = time.time() + 1
    while time.time() < stop:
        interpret(code1)
        interpret(code2)
    #
    rvmprof.disable()
    os.close(fd)
    return 0

# ____________________________________________________________

def target(driver, args):
    return main

def test_interpreted():
    # takes forever if the Python process is already big...
    import subprocess
    me = os.path.basename(__file__)
    if me.endswith('pyc') or me.endswith('pyo'):
        me = me[:-1]
    env = os.environ.copy()
    env['PYTHONPATH'] = ''
    subprocess.check_call([sys.executable, me],
                          cwd=(os.path.dirname(__file__) or '.'),
                          env=env)

def test_compiled():
    fn = compile(main, [], gcpolicy="minimark")
    if os.path.exists(PROF_FILE):
        os.unlink(PROF_FILE)
    fn()
    assert os.path.exists(PROF_FILE)

if __name__ == '__main__':
    setup_module(None)
    res = main()
    assert res == 0



from rpython.rlib.cache import Cache 

class MyCache(Cache):
    counter = 0
    def _build(self, key):
        self.counter += 1
        return key*7

class TestCache: 
    def test_getorbuild(self):
        cache = MyCache()
        assert cache.getorbuild(1) == 7
        assert cache.counter == 1
        assert cache.getorbuild(1) == 7
        assert cache.counter == 1
        assert cache.getorbuild(3) == 21
        assert cache.counter == 2
        assert cache.getorbuild(1) == 7
        assert cache.counter == 2
        assert cache.getorbuild(3) == 21
        assert cache.counter == 2



# This is the default unicodedb used in various places:
# - the unicode type
# - the regular expression engine
from rpython.rlib.unicodedata import unicodedb_5_2_0 as unicodedb

# to get information about individual unicode chars look at:
# http://www.fileformat.info/info/unicode/char/search.htm

# the following externally usable functions are defined in each of the
# unicodedb files.

# all code arguments are integers


"""
lookup(name, with_named_sequence=False, with_alias=False)
    Look up character by name.

    If a character with the given name is found, return the
    corresponding character.  If not found, KeyError is raised.

    If with_named_sequence is True, named sequences are also searched. They are
    returned into the form of a code point in a private use area and must be
    converted with lookup_named_sequence and lookup_named_sequence_length

    with_alias should not be set to True, use lookup_with_alias instead.

lookup_with_alias(name, with_named_sequence=False)
    Like lookup, but will also search the name aliases.

lookup_named_sequence(code)
    Turns a code point that is the result of lookup or lookup_with_alias with
    argument with_named_sequence=True and returns a utf-8 encoded unicode
    string, if the code point is in the right range of the private use area
    that lookup uses for named sequences. Otherwise None is returned.

lookup_named_sequence_length(code)
    Like lookup_named_sequence, but returns the number of code points of the
    named sequence that code represents.

name(code)
    Returns the name assigned to the character code as a string.
    If no name is defined, KeyError is raised.

composition(current, next)
    Composes two code points current and next into a new one. Raises KeyError
    if that's not possible.

decomposition(code)
    Returns the character decomposition mapping assigned to the character code
    as string.
    An empty string is returned in case no such mapping is defined.

canon_decomposition(code)
    Returns the canonical decomposition of code as a list of integers.
    Returns an empty list if the code point is not decomposable.

compat_decomposition(code)
    Returns the compatibility decomposition of code as a list of integers.
    Returns an empty list if the code point is not decomposable.

combining(code)
    Returns the canonical combining class assigned to the character code as integer.
    Returns 0 if no combining class is defined.

category(code)
    Returns the general category assigned to the character code as string.

bidirectional(code)
    Returns the bidirectional class assigned to the character code as string.
    If no such value is defined, an empty string is returned.

east_asian_width(code)
    Returns the east asian width assigned to the character code as string.


The following functions all check various properties of characters:

isspace(code)
isalpha(code)
islinebreak(code)
isnumeric(code)
isdigit(code)
isdecimal(code)
isalnum(code)
isupper(code)
istitle(code)
islower(code)
iscased(code)
isxidstart(code)
isxidcontinue(code)
isprintable(code)
iscaseignorable(code)

mirrored(code)
    Returns the mirrored property assigned to the character code as integer.

decimal(code)
    Converts a Unicode character into its equivalent decimal value.

    Returns the decimal value assigned to the character code as integer.
    Otherwise a KeyError is raised.

digit(code)
    Converts a Unicode character into its equivalent digit value.

    Returns the digit value assigned to the character code as integer.
    Otherwise a KeyError is raised.

numeric(code)
    Converts a Unicode character into its equivalent numeric value.

    Returns the numeric value assigned to the character code as float.
    Otherwise a KeyError is raised.

toupper(code)
    Return the upper case version of the code point, or just the argument
    itself, if it is not cased. This uses the "simple" case folding rules,
    where every code point maps to one code point only.

tolower(code)
    Return the lower case version of the code point, or just the argument
    itself, if it is not cased. This uses the "simple" case folding rules,
    where every code point maps to one code point only.

totitle(code)
    Return the title case version of the code point, or just the argument
    itself, if it is not cased. This uses the "simple" case folding rules,
    where every code point maps to one code point only.

toupper_full(code)
    Return the upper case version of the code point, or just the argument
    itself, if it is not cased. This uses the "full" case folding rules,
    thus it returns a list of code points.

tolower_full(code)
    Return the lower case version of the code point, or just the argument
    itself, if it is not cased. This uses the "full" case folding rules,
    thus it returns a list of code points.

totitle_full(code)
    Return the title case version of the code point, or just the argument
    itself, if it is not cased. This uses the "full" case folding rules,
    thus it returns a list of code points.

casefold_lookup(code)
    Return a version of the code point suitable for caseless
    comparisons. Returns a list of code points.

"""



from rpython.rtyper.lltypesystem import lltype
from rpython.rtyper.lltypesystem.lloperation import llop
from rpython.rtyper.extregistry import ExtRegistryEntry
from rpython.annotator import model as annmodel
import os


def find_gc_ptrs_in_type(TYPE):
    if isinstance(TYPE, lltype.Array):
        return find_gc_ptrs_in_type(TYPE.OF)
    elif isinstance(TYPE, lltype.Struct):
        result = []
        for name in TYPE._names:
            result.extend(find_gc_ptrs_in_type(TYPE._flds[name]))
        return result
    elif isinstance(TYPE, lltype.Ptr) and TYPE._needsgc():
        return [TYPE]
    elif isinstance(TYPE, lltype.GcOpaqueType):
        # heuristic: in theory the same problem exists with OpaqueType, but
        # we use OpaqueType for other things too that we know are safely
        # empty of further gc pointers
        raise Exception("don't know what is in %r" % (TYPE,))
    else:
        return []

def get_rtti(TYPE):
    if isinstance(TYPE, lltype.RttiStruct):
        try:
            return lltype.getRuntimeTypeInfo(TYPE)
        except ValueError:
            pass
    return None

def _static_deallocator_body_for_type(v, TYPE, depth=1):
    if isinstance(TYPE, lltype.Array):
        inner = list(_static_deallocator_body_for_type('v_%i'%depth, TYPE.OF, depth+1))
        if inner:
            yield '    '*depth + 'i_%d = 0'%(depth,)
            yield '    '*depth + 'l_%d = len(%s)'%(depth, v)
            yield '    '*depth + 'while i_%d < l_%d:'%(depth, depth)
            yield '    '*depth + '    v_%d = %s[i_%d]'%(depth, v, depth)
            for line in inner:
                yield line
            yield '    '*depth + '    i_%d += 1'%(depth,)
    elif isinstance(TYPE, lltype.Struct):
        for name in TYPE._names:
            inner = list(_static_deallocator_body_for_type(
                v + '_' + name, TYPE._flds[name], depth))
            if inner:
                yield '    '*depth + v + '_' + name + ' = ' + v + '.' + name
                for line in inner:
                    yield line
    elif isinstance(TYPE, lltype.Ptr) and TYPE._needsgc():
        yield '    '*depth + 'pop_alive(%s)'%v

class LLTransformerOp(object):
    """Objects that can be called in ll functions.
    Their calls are replaced by a simple operation of the GC transformer,
    e.g. ll_pop_alive.
    """
    def __init__(self, transformer_method):
        self.transformer_method = transformer_method

class LLTransformerOpEntry(ExtRegistryEntry):
    "Annotation and specialization of LLTransformerOp() instances."
    _type_ = LLTransformerOp

    def compute_result_annotation(self, s_arg):
        return annmodel.s_None

    def specialize_call(self, hop):
        op = self.instance   # the LLTransformerOp instance
        op.transformer_method(hop.args_v[0], hop.llops)
        hop.exception_cannot_occur()
        return hop.inputconst(hop.r_result.lowleveltype, hop.s_result.const)

def write(fd, string):
    from rpython.rlib.rposix import c_write
    return c_write(fd, string, len(string))

def ll_call_destructor(destrptr, destr_v, typename):
    llop.revdb_do_next_call(lltype.Void)
    try:
        destrptr(destr_v)
    except Exception as e:
        try:
            write(2, "a destructor of type ")
            write(2, typename)
            write(2, " raised an exception ")
            write(2, str(e))
            write(2, " ignoring it\n")
        except:
            pass

def ll_report_finalizer_error(e):
    try:
        write(2, "triggering finalizers raised an exception ")
        write(2, str(e))
        write(2, " ignoring it\n")
    except:
        pass



import os

def main(iterations=1):
    source = os.open('/dev/zero', os.O_RDWR, 0777)
    
    for x in xrange(1024 * 1024 * iterations):
        payload = os.read(source, 1024)

    os.close(source)
    
def entry_point(argv):
    if len(argv) > 1:
        n = int(argv[1])
    else:
        n = 1
    main(n)
    return 0

# _____ Define and setup target ___

def target(*args):
    return entry_point, None

if __name__ == '__main__':
    import sys
    if len(sys.argv) >= 2:
        main(iterations = int(sys.argv[1]))
    else:
        main()



from pypy.module.pypyjit import policy

pypypolicy = policy.PyPyJitPolicy()

def test_id_any():
    from pypy.objspace.std.intobject import W_IntObject
    assert pypypolicy.look_inside_function(W_IntObject.descr_add)

def test_rlocale():
    from rpython.rlib.rlocale import setlocale
    assert not pypypolicy.look_inside_function(setlocale)

def test_astcompiler():
    from pypy.interpreter.astcompiler import ast
    assert not pypypolicy.look_inside_function(ast.AST.walkabout)

def test_pyparser():
    from pypy.interpreter.pyparser import parser
    assert not pypypolicy.look_inside_function(parser.Grammar.__init__.im_func)

def test_property():
    from pypy.module.__builtin__.descriptor import W_Property
    assert pypypolicy.look_inside_function(W_Property.get.im_func)

def test_thread_local():
    from pypy.module.thread.os_local import Local
    from pypy.module.thread.os_thread import get_ident
    assert pypypolicy.look_inside_function(Local.getdict.im_func)
    assert pypypolicy.look_inside_function(get_ident)

def test_time():
    from pypy.module.time.interp_time import time
    assert pypypolicy.look_inside_function(time)

def test_io():
    from pypy.module._io.interp_bytesio import W_BytesIO
    assert pypypolicy.look_inside_function(W_BytesIO.seek_w.im_func)

def test_thread():
    from pypy.module.thread.os_lock import Lock
    assert pypypolicy.look_inside_function(Lock.descr_lock_acquire.im_func)

def test_select():
    from pypy.module.select.interp_select import poll
    assert pypypolicy.look_inside_function(poll)

def test_pypy_module():
    from pypy.module._collections.interp_deque import W_Deque
    from pypy.module._random.interp_random import W_Random
    assert pypypolicy.look_inside_function(W_Random.random)
    assert pypypolicy.look_inside_function(W_Deque.length)
    assert pypypolicy.look_inside_pypy_module('__builtin__.operation')
    assert pypypolicy.look_inside_pypy_module('__builtin__.abstractinst')
    assert pypypolicy.look_inside_pypy_module('__builtin__.functional')
    assert pypypolicy.look_inside_pypy_module('__builtin__.descriptor')
    assert pypypolicy.look_inside_pypy_module('exceptions.interp_exceptions')
    for modname in 'pypyjit', 'signal', 'micronumpy', 'math', 'imp':
        assert pypypolicy.look_inside_pypy_module(modname)
        assert pypypolicy.look_inside_pypy_module(modname + '.foo')
    assert not pypypolicy.look_inside_pypy_module('pypyjit.interp_resop')

def test_see_jit_module():
    assert pypypolicy.look_inside_pypy_module('pypyjit.interp_jit')



import py

from rpython.jit.metainterp.optimizeopt.util import equaloplists
from rpython.jit.metainterp.optimizeopt.vector import (
    GenericCostModel, NotAProfitableLoop, VectorizingOptimizer, CostModel)
from rpython.jit.metainterp.optimizeopt.schedule import VecScheduleState
from rpython.jit.metainterp.optimizeopt.dependency import DependencyGraph
from rpython.jit.metainterp.optimizeopt.test.test_schedule import SchedulerBaseTest
from rpython.jit.metainterp.optimizeopt.test.test_vecopt import (
    FakeJitDriverStaticData)

class FakeMemoryRef(object):
    def __init__(self, array, iv):
        self.index_var = iv
        self.array = array

    def is_adjacent_after(self, other):
        if self.array is not other.array:
            return False
        iv = self.index_var
        ov = other.index_var
        val = (int(str(ov.var)[1:]) - int(str(iv.var)[1:]))
        # i0 and i1 are adjacent
        # i1 and i0 ...
        # but not i0, i2
        # ...
        #print iv, 'is after', ov, "?", val == 1
        return val == 1

def prepost_savings(orig_func):
    def func(self, *args):
        f = getattr(self.proxy, orig_func.__name__)
        before_savings = self.proxy.savings
        r = f(*args)
        after_savings = self.proxy.savings
        print " CM %s (%d -> %d, diff: %d) " % (orig_func.__name__,
              before_savings, after_savings,
              (after_savings - before_savings),)
        print " args: ", args
        return r
    return func

class FakeCostModel(CostModel):
    def __init__(self, proxy):
        self.proxy = proxy

    def getsavings(self):
        return self.proxy.savings

    @prepost_savings
    def reset_savings(self):
        raise NotImplementedError

    @prepost_savings
    def record_cast_int(self, op):
        raise NotImplementedError

    @prepost_savings
    def record_pack_savings(self, pack, times):
        raise NotImplementedError

    @prepost_savings
    def record_vector_pack(self, box, index, count):
        raise NotImplementedError

    @prepost_savings
    def record_vector_unpack(self, box, index, count):
        raise NotImplementedError

    @prepost_savings
    def unpack_cost(self, op, index, count):
        raise NotImplementedError

    @prepost_savings
    def savings_for_pack(self, pack, times):
        raise NotImplementedError

    def profitable(self):
        return self.proxy.savings >= 0

class TestCostModel(SchedulerBaseTest):
    def savings(self, loop):
        jitdriver_sd = FakeJitDriverStaticData()
        opt = VectorizingOptimizer(self.metainterp_sd, jitdriver_sd, 0)
        opt.orig_label_args = loop.label.getarglist()[:]
        graph = opt.dependency_graph = DependencyGraph(loop)
        self.show_dot_graph(graph, 'costmodel')
        for k, m in graph.memory_refs.items():
            graph.memory_refs[k] = FakeMemoryRef(m.array, m.index_var)
        opt.find_adjacent_memory_refs(graph)
        opt.extend_packset()
        opt.combine_packset()
        for pack in opt.packset.packs:
            print "pack: \n   ",
            print '\n    '.join([str(op.getoperation()) for op in pack.operations])
            print
        costmodel = FakeCostModel(GenericCostModel(self.cpu, 0))
        costmodel.reset_savings()
        state = VecScheduleState(graph, opt.packset, self.cpu, costmodel)
        opt.schedule(state)
        return costmodel.getsavings()

    def assert_operations_match(self, loop_a, loop_b):
        assert equaloplists(loop_a.operations, loop_b.operations)

    def test_load_2_unpack(self):
        loop1 = self.parse_trace("""
        f10 = raw_load_f(p0, i0, descr=double)
        f11 = raw_load_f(p0, i1, descr=double)
        guard_true(i0) [f10]
        guard_true(i1) [f11]
        """)
        # for double the costs are
        # unpack index 1 savings: -2
        # unpack index 0 savings: -1
        savings = self.savings(loop1)
        assert savings == -2

    def test_load_4_unpack(self):
        loop1 = self.parse_trace("""
        i10 = raw_load_i(p0, i0, descr=float)
        i11 = raw_load_i(p0, i1, descr=float)
        i12 = raw_load_i(p0, i2, descr=float)
        i13 = raw_load_i(p0, i3, descr=float)
        guard_true(i0) [i10]
        guard_true(i1) [i11]
        guard_true(i2) [i12]
        guard_true(i3) [i13]
        """)
        savings = self.savings(loop1)
        assert savings == -1

    def test_load_2_unpack_1(self):
        loop1 = self.parse_trace("""
        f10 = raw_load_f(p0, i0, descr=double)
        f11 = raw_load_f(p0, i1, descr=double)
        guard_true(i0) [f10]
        """)
        assert loop1.operations[2].getfailargs()[0] is loop1.operations[0]
        savings = self.savings(loop1)
        assert savings == 0
        assert loop1.operations[2].getfailargs()[0] is loop1.operations[-2]

    def test_load_2_unpack_1_index1(self):
        loop1 = self.parse_trace("""
        f10 = raw_load_f(p0, i0, descr=double)
        f11 = raw_load_f(p0, i1, descr=double)
        guard_true(i0) [f11]
        """)
        savings = self.savings(loop1)
        assert savings == -1

    def test_load_arith1(self):
        loop1 = self.parse_trace("""
        i10 = raw_load_i(p0, i0, descr=int)
        i11 = raw_load_i(p0, i1, descr=int)
        i12 = raw_load_i(p0, i2, descr=int)
        i13 = raw_load_i(p0, i3, descr=int)
        i15 = int_add(i10, 1)
        i16 = int_add(i11, 1)
        i17 = int_add(i12, 1)
        i18 = int_add(i13, 1)
        """)
        savings = self.savings(loop1)
        assert savings == 6

    def test_load_arith_store(self):
        import platform
        size = 4
        if not platform.machine().startswith('x86'):
            size = 8
        loop1 = self.parse_trace("""
        f10 = raw_load_f(p0, i0, descr=double)
        f11 = raw_load_f(p0, i1, descr=double)
        i20 = cast_float_to_int(f10)
        i21 = cast_float_to_int(f11)
        i30 = int_signext(i20, {size})
        i31 = int_signext(i21, {size})
        raw_store(p0, i3, i30, descr=int)
        raw_store(p0, i4, i31, descr=int)
        """.format(size=size))
        savings = self.savings(loop1)
        assert savings >= 0

    def test_sum(self):
        loop1 = self.parse_trace("""
        f10 = raw_load_f(p0, i0, descr=double)
        f11 = raw_load_f(p0, i1, descr=double)
        f12 = float_add(f1, f10)
        f13 = float_add(f12, f11)
        """)
        savings = self.savings(loop1)
        assert savings == -2

    @py.test.mark.parametrize("bytes,s", [(4,0),(8,0)])
    def test_sum_float_to_int(self, bytes, s):
        loop1 = self.parse_trace("""
        f10 = raw_load_f(p0, i0, descr=double)
        f11 = raw_load_f(p0, i1, descr=double)
        i10 = cast_float_to_int(f10)
        i11 = cast_float_to_int(f11)
        i12 = int_signext(i10, {c})
        i13 = int_signext(i11, {c})
        i14 = int_add(i1, i12)
        i16 = int_signext(i14, {c})
        i15 = int_add(i16, i13)
        i17 = int_signext(i15, {c})
        """.format(c=bytes))
        try:
            savings = self.savings(loop1)
            if s is None:
                py.test.fail("must fail")
            # it does not benefit because signext has
            # a very inefficient implementation (x86
            # does not provide nice instr to convert
            # integer sizes)
            # signext -> no benefit, + 2x unpack
            assert savings <= s
        except NotAProfitableLoop:
            if s is not None:
                py.test.fail("must not fail")

    def test_cast(self):
        loop1 = self.parse_trace("""
        i100 = raw_load_i(p0, i1, descr=float)
        i101 = raw_load_i(p0, i2, descr=float)
        i102 = raw_load_i(p0, i3, descr=float)
        i103 = raw_load_i(p0, i4, descr=float)
        #
        i104 = raw_load_i(p1, i1, descr=short)
        i105 = raw_load_i(p1, i2, descr=short)
        i106 = raw_load_i(p1, i3, descr=short)
        i107 = raw_load_i(p1, i4, descr=short)
        i108 = raw_load_i(p1, i5, descr=short)
        i109 = raw_load_i(p1, i6, descr=short)
        i110 = raw_load_i(p1, i7, descr=short)
        i111 = raw_load_i(p1, i8, descr=short)
        #
        f100 = cast_int_to_float(i104)
        f101 = cast_int_to_float(i105)
        f102 = cast_int_to_float(i106)
        f103 = cast_int_to_float(i107)
        f104 = cast_int_to_float(i108)
        f105 = cast_int_to_float(i109)
        f106 = cast_int_to_float(i110)
        f107 = cast_int_to_float(i111)
        """)
        try:
            self.savings(loop1)
            py.test.fail("must not be profitable!")
        except NotAProfitableLoop:
            pass

    def test_force_int_to_float_cast(self):
        trace = self.parse_trace("""
        i10 = raw_load_i(p0, i1, descr=long)
        i11 = raw_load_i(p0, i2, descr=long)
        f10 = cast_int_to_float(i10)
        f11 = cast_int_to_float(i11)
        """)
        number = self.savings(trace)
        assert number >= 1



from rpython.jit.backend.arm.helper.assembler import count_reg_args
from rpython.jit.metainterp.history import INT, REF, FLOAT
from rpython.jit.metainterp.resoperation import InputArgInt as BoxInt
from rpython.jit.metainterp.resoperation import InputArgRef as BoxPtr
from rpython.jit.metainterp.resoperation import InputArgFloat as BoxFloat


def test_count_reg_args():
    assert count_reg_args([BoxPtr()]) == 1
    assert count_reg_args([BoxPtr()] * 2) == 2
    assert count_reg_args([BoxPtr()] * 3) == 3
    assert count_reg_args([BoxPtr()] * 4) == 4
    assert count_reg_args([BoxPtr()] * 5) == 4
    assert count_reg_args([BoxFloat()] * 1) == 1
    assert count_reg_args([BoxFloat()] * 2) == 2
    assert count_reg_args([BoxFloat()] * 3) == 2

    assert count_reg_args([BoxInt(), BoxInt(), BoxFloat()]) == 3
    assert count_reg_args([BoxInt(), BoxFloat(), BoxInt()]) == 2

    assert count_reg_args([BoxInt(), BoxFloat(), BoxInt()]) == 2
    assert count_reg_args([BoxInt(), BoxInt(), BoxInt(), BoxFloat()]) == 3



import py

from pypy.module.__pypy__.interp_time import HAS_CLOCK_GETTIME_RUNTIME


class AppTestTime(object):
    def setup_class(cls):
        if not HAS_CLOCK_GETTIME_RUNTIME:
            py.test.skip("need time.clock_gettime")

    def test_clock_realtime(self):
        from __pypy__ import time
        res = time.clock_gettime(time.CLOCK_REALTIME)
        assert isinstance(res, float)

    def test_clock_monotonic(self):
        from __pypy__ import time
        a = time.clock_gettime(time.CLOCK_MONOTONIC)
        b = time.clock_gettime(time.CLOCK_MONOTONIC)
        assert a <= b

    def test_clock_getres(self):
        from __pypy__ import time
        res = time.clock_getres(time.CLOCK_REALTIME)
        assert res > 0.0
        assert res <= 1.0



from rpython.jit.metainterp.test import test_longlong
from rpython.jit.backend.x86.test.test_basic import Jit386Mixin

class TestLongLong(Jit386Mixin, test_longlong.LongLongTests):
    # for the individual tests see
    # ====> ../../../metainterp/test/test_longlong.py
    pass



from pypy.interpreter.error import OperationError, oefmt
from pypy.interpreter.typedef import TypeDef
from pypy.interpreter.gateway import interp2app
from pypy.interpreter.baseobjspace import W_Root


class W_IdentityDict(W_Root):
    def __init__(self, space):
        self.dict = {}

    def descr_new(space, w_subtype):
        self = space.allocate_instance(W_IdentityDict, w_subtype)
        W_IdentityDict.__init__(self, space)
        return self

    def descr_len(self, space):
        return space.newint(len(self.dict))

    def descr_contains(self, space, w_key):
        return space.newbool(w_key in self.dict)

    def descr_setitem(self, space, w_key, w_value):
        self.dict[w_key] = w_value

    def descr_getitem(self, space, w_key):
        try:
            return self.dict[w_key]
        except KeyError:
            raise OperationError(space.w_KeyError, w_key)

    def descr_delitem(self, space, w_key):
        try:
            del self.dict[w_key]
        except KeyError:
            raise OperationError(space.w_KeyError, w_key)

    def descr_iter(self, space):
        raise oefmt(space.w_TypeError,
                    "'identity_dict' object does not support iteration; "
                    "iterate over x.keys()")

    def get(self, space, w_key, w_default=None):
        if w_default is None:
            w_default = space.w_None
        return self.dict.get(w_key, w_default)

    def keys(self, space):
        return space.newlist(self.dict.keys())

    def values(self, space):
        return space.newlist(self.dict.values())

    def clear(self, space):
        self.dict.clear()

W_IdentityDict.typedef = TypeDef("identity_dict",
    __doc__="""\
A dictionary that considers keys by object identity.
Distinct objects will have separate entries even if they
compare equal.  All objects can be used as keys, even
non-hashable ones --- but avoid using immutable objects
like integers: two int objects 42 may or may not be
internally the same object.
""",
    __new__ = interp2app(W_IdentityDict.descr_new.im_func),
    __len__ = interp2app(W_IdentityDict.descr_len),
    __contains__ = interp2app(W_IdentityDict.descr_contains),
    __setitem__ = interp2app(W_IdentityDict.descr_setitem),
    __getitem__ = interp2app(W_IdentityDict.descr_getitem),
    __delitem__ = interp2app(W_IdentityDict.descr_delitem),
    __iter__ = interp2app(W_IdentityDict.descr_iter),
    get = interp2app(W_IdentityDict.get),
    keys = interp2app(W_IdentityDict.keys),
    values = interp2app(W_IdentityDict.values),
    clear = interp2app(W_IdentityDict.clear),
)



from rpython.annotator.model import s_ImpossibleValue
from rpython.annotator.model import SomeList, SomeString
from rpython.annotator.model import unionof, TLS, UnionError, AnnotatorError


class TooLateForChange(AnnotatorError):
    pass

class ListChangeUnallowed(AnnotatorError):
    pass

class ListItem(object):
    mutated = False    # True for lists mutated after creation
    resized = False    # True for lists resized after creation
    range_step = None  # the step -- only for lists only created by a range()
    dont_change_any_more = False   # set to True when too late for changes
    immutable = False  # for getattr out of _immutable_fields_ = ['attr[*]']
    must_not_resize = False   # make_sure_not_resized()

    # what to do if range_step is different in merge.
    # - if one is a list (range_step is None), unify to a list.
    # - if both have a step, unify to use a variable step (indicated by 0)
    _step_map = {
        (type(None), int): None,
        (int, type(None)): None,
        (int, int)       : 0,
        }

    def __init__(self, bookkeeper, s_value):
        self.s_value = s_value
        self.bookkeeper = bookkeeper
        self.itemof = {}  # set of all ListDefs using this ListItem
        self.read_locations = set()
        if bookkeeper is None:
            self.dont_change_any_more = True

    def mutate(self):
        if not self.mutated:
            if self.dont_change_any_more:
                raise TooLateForChange
            self.immutable = False
            self.mutated = True

    def resize(self):
        if not self.resized:
            if self.dont_change_any_more:
                raise TooLateForChange
            if self.must_not_resize:
                raise ListChangeUnallowed("resizing list")
            self.resized = True

    def setrangestep(self, step):
        if step != self.range_step:
            if self.dont_change_any_more:
                raise TooLateForChange
            self.range_step = step

    def merge(self, other):
        if self is not other:
            if getattr(TLS, 'no_side_effects_in_union', 0):
                raise UnionError(self, other)

            if other.dont_change_any_more:
                if self.dont_change_any_more:
                    raise TooLateForChange
                else:
                    # lists using 'other' don't expect it to change any more,
                    # so we try merging into 'other', which will give
                    # TooLateForChange if it actually tries to make
                    # things more general
                    self, other = other, self

            self.immutable &= other.immutable
            if other.must_not_resize:
                if self.resized:
                    raise ListChangeUnallowed("list merge with a resized")
                self.must_not_resize = True
            if other.mutated:
                self.mutate()
            if other.resized:
                self.resize()
            if other.range_step != self.range_step:
                self.setrangestep(self._step_map[type(self.range_step),
                                                 type(other.range_step)])
            self.itemof.update(other.itemof)
            s_value = self.s_value
            s_other_value = other.s_value
            s_new_value = unionof(s_value, s_other_value)
            if s_new_value != s_value:
                if self.dont_change_any_more:
                    raise TooLateForChange
            self.patch()    # which should patch all refs to 'other'
            if s_new_value != s_value:
                self.s_value = s_new_value
                self.notify_update()
            if s_new_value != s_other_value:
                other.notify_update()
            self.read_locations |= other.read_locations

    def patch(self):
        for listdef in self.itemof:
            listdef.listitem = self

    def notify_update(self):
        '''Reflow from all reading points'''
        for position_key in self.read_locations:
            self.bookkeeper.annotator.reflowfromposition(position_key)

    def generalize(self, s_other_value):
        s_new_value = unionof(self.s_value, s_other_value)
        updated = s_new_value != self.s_value
        if updated:
            if self.dont_change_any_more:
                raise TooLateForChange
            self.s_value = s_new_value
            self.notify_update()
        return updated


class ListDef(object):
    """A list definition remembers how general the items in that particular
    list have to be.  Every list creation makes a new ListDef, and the union
    of two lists merges the ListItems that each ListDef stores."""

    def __init__(self, bookkeeper, s_item=s_ImpossibleValue,
                 mutated=False, resized=False):
        self.listitem = ListItem(bookkeeper, s_item)
        self.listitem.mutated = mutated | resized
        self.listitem.resized = resized
        self.listitem.itemof[self] = True

    def read_item(self, position_key):
        self.listitem.read_locations.add(position_key)
        return self.listitem.s_value

    def same_as(self, other):
        return self.listitem is other.listitem

    def union(self, other):
        self.listitem.merge(other.listitem)
        return self

    def agree(self, bookkeeper, other):
        position = bookkeeper.position_key
        s_self_value = self.read_item(position)
        s_other_value = other.read_item(position)
        self.generalize(s_other_value)
        other.generalize(s_self_value)
        if self.listitem.range_step is not None:
            self.generalize_range_step(other.listitem.range_step)
        if other.listitem.range_step is not None:
            other.generalize_range_step(self.listitem.range_step)

    def offspring(self, bookkeeper, *others):
        position = bookkeeper.position_key
        s_self_value = self.read_item(position)
        s_other_values = []
        for other in others:
            s_other_values.append(other.read_item(position))
        s_newlst = bookkeeper.newlist(s_self_value, *s_other_values)
        s_newvalue = s_newlst.listdef.read_item(position)
        self.generalize(s_newvalue)
        for other in others:
            other.generalize(s_newvalue)
        return s_newlst

    def generalize(self, s_value):
        self.listitem.generalize(s_value)

    def generalize_range_step(self, range_step):
        newlistitem = ListItem(self.listitem.bookkeeper, s_ImpossibleValue)
        newlistitem.range_step = range_step
        self.listitem.merge(newlistitem)

    def __repr__(self):
        return '<[%r]%s%s%s%s>' % (self.listitem.s_value,
                               self.listitem.mutated and 'm' or '',
                               self.listitem.resized and 'r' or '',
                               self.listitem.immutable and 'I' or '',
                               self.listitem.must_not_resize and '!R' or '')

    def mutate(self):
        self.listitem.mutate()

    def resize(self):
        self.listitem.mutate()
        self.listitem.resize()

    def never_resize(self):
        if self.listitem.resized:
            raise ListChangeUnallowed("list already resized")
        self.listitem.must_not_resize = True

    def mark_as_immutable(self):
        # Sets the 'immutable' flag.  Note that unlike "never resized",
        # the immutable flag is only a hint.  It is cleared again e.g.
        # when we merge with a "normal" list that doesn't have it.  It
        # is thus expected to live only shortly, mostly for the case
        # of writing 'x.list[n]'.
        self.never_resize()
        if not self.listitem.mutated:
            self.listitem.immutable = True
        #else: it's fine, don't set immutable=True at all (see
        #      test_can_merge_immutable_list_with_regular_list)

s_list_of_strings = SomeList(ListDef(None, SomeString(no_nul=True),
                                     resized = True))



"""
Function pointers.
"""

import sys

from rpython.rlib import jit, clibffi, jit_libffi, rgc
from rpython.rlib.jit_libffi import (CIF_DESCRIPTION, CIF_DESCRIPTION_P,
    FFI_TYPE, FFI_TYPE_P, FFI_TYPE_PP, SIZE_OF_FFI_ARG)
from rpython.rlib.objectmodel import we_are_translated, instantiate
from rpython.rlib.objectmodel import keepalive_until_here
from rpython.rtyper.lltypesystem import lltype, llmemory, rffi
from rpython.rtyper.annlowlevel import llstr

from pypy.interpreter.error import OperationError, oefmt
from pypy.module._cffi_backend.moduledef import (
    FFI_DEFAULT_ABI, has_stdcall, FFI_STDCALL)
from pypy.module._cffi_backend import ctypearray, cdataobj, cerrno
from pypy.module._cffi_backend.ctypeobj import W_CType
from pypy.module._cffi_backend.ctypeptr import W_CTypePtrBase, W_CTypePointer
from pypy.module._cffi_backend.ctypevoid import W_CTypeVoid
from pypy.module._cffi_backend.ctypestruct import W_CTypeStruct, W_CTypeUnion
from pypy.module._cffi_backend.ctypeprim import (W_CTypePrimitiveSigned,
    W_CTypePrimitiveUnsigned, W_CTypePrimitiveCharOrUniChar,
    W_CTypePrimitiveFloat, W_CTypePrimitiveLongDouble, W_CTypePrimitiveComplex)


class W_CTypeFunc(W_CTypePtrBase):
    _attrs_            = ['fargs', 'ellipsis', 'abi', 'cif_descr']
    _immutable_fields_ = ['fargs[*]', 'ellipsis', 'abi', 'cif_descr']
    kind = "function"

    cif_descr = lltype.nullptr(CIF_DESCRIPTION)

    def __init__(self, space, fargs, fresult, ellipsis,
                 abi=FFI_DEFAULT_ABI):
        assert isinstance(ellipsis, bool)
        extra, xpos = self._compute_extra_text(fargs, fresult, ellipsis, abi)
        size = rffi.sizeof(rffi.VOIDP)
        W_CTypePtrBase.__init__(self, space, size, extra, xpos, fresult)
        self.fargs = fargs
        self.ellipsis = ellipsis
        self.abi = abi
        # fresult is stored in self.ctitem

        if not ellipsis:
            # Functions with '...' varargs are stored without a cif_descr
            # at all.  The cif is computed on every call from the actual
            # types passed in.  For all other functions, the cif_descr
            # is computed here.
            builder = CifDescrBuilder(fargs, fresult, abi)
            try:
                builder.rawallocate(self)
            except OperationError as e:
                if not e.match(space, space.w_NotImplementedError):
                    raise
                # else, eat the NotImplementedError.  We will get the
                # exception if we see an actual call
                if self.cif_descr:   # should not be True, but you never know
                    lltype.free(self.cif_descr, flavor='raw')
                    self.cif_descr = lltype.nullptr(CIF_DESCRIPTION)

    def is_unichar_ptr_or_array(self):
        return False

    def is_char_or_unichar_ptr_or_array(self):
        return False

    def string(self, cdataobj, maxlen):
        # Can't use ffi.string() on a function pointer
        return W_CType.string(self, cdataobj, maxlen)

    def new_ctypefunc_completing_argtypes(self, args_w):
        space = self.space
        nargs_declared = len(self.fargs)
        fvarargs = [None] * len(args_w)
        for i in range(nargs_declared):
            fvarargs[i] = self.fargs[i]
        for i in range(nargs_declared, len(args_w)):
            w_obj = args_w[i]
            if isinstance(w_obj, cdataobj.W_CData):
                ct = w_obj.ctype.get_vararg_type()
            else:
                raise oefmt(space.w_TypeError,
                            "argument %d passed in the variadic part needs to "
                            "be a cdata object (got %T)", i + 1, w_obj)
            fvarargs[i] = ct
        # xxx call instantiate() directly.  It's a bit of a hack.
        ctypefunc = instantiate(W_CTypeFunc)
        ctypefunc.space = space
        ctypefunc.fargs = fvarargs
        ctypefunc.ctitem = self.ctitem
        #ctypefunc.cif_descr = NULL --- already provided as the default
        CifDescrBuilder(fvarargs, self.ctitem, self.abi).rawallocate(ctypefunc)
        return ctypefunc

    @rgc.must_be_light_finalizer
    def __del__(self):
        if self.cif_descr:
            lltype.free(self.cif_descr, flavor='raw')

    def _compute_extra_text(self, fargs, fresult, ellipsis, abi):
        argnames = ['(*)(']
        xpos = 2
        if has_stdcall and abi == FFI_STDCALL:
            argnames[0] = '(__stdcall *)('
            xpos += len('__stdcall ')
        for i, farg in enumerate(fargs):
            if i > 0:
                argnames.append(', ')
            argnames.append(farg.name)
        if ellipsis:
            if len(fargs) > 0:
                argnames.append(', ')
            argnames.append('...')
        argnames.append(')')
        return ''.join(argnames), xpos

    def _fget(self, attrchar):
        if attrchar == 'a':    # args
            return self.space.newtuple([a for a in self.fargs])
        if attrchar == 'r':    # result
            return self.ctitem
        if attrchar == 'E':    # ellipsis
            return self.space.newbool(self.ellipsis)
        if attrchar == 'A':    # abi
            return self.space.newint(self.abi)
        return W_CTypePtrBase._fget(self, attrchar)

    def call(self, funcaddr, args_w):
        if not funcaddr:
            raise oefmt(self.space.w_RuntimeError,
                        "cannot call null function pointer from cdata '%s'",
                        self.name)
        if self.cif_descr:
            # regular case: this function does not take '...' arguments
            self = jit.promote(self)
            nargs_declared = len(self.fargs)
            if len(args_w) != nargs_declared:
                space = self.space
                raise oefmt(space.w_TypeError,
                            "'%s' expects %d arguments, got %d",
                            self.name, nargs_declared, len(args_w))
            return self._call(funcaddr, args_w)
        else:
            # call of a variadic function
            return self.call_varargs(funcaddr, args_w)

    @jit.dont_look_inside
    def call_varargs(self, funcaddr, args_w):
        nargs_declared = len(self.fargs)
        if len(args_w) < nargs_declared:
            space = self.space
            raise oefmt(space.w_TypeError,
                        "'%s' expects at least %d arguments, got %d",
                        self.name, nargs_declared, len(args_w))
        completed = self.new_ctypefunc_completing_argtypes(args_w)
        return completed._call(funcaddr, args_w)

    # The following is the core of function calls.  It is @unroll_safe,
    # which means that the JIT is free to unroll the argument handling.
    # But in case the function takes variable arguments, we don't unroll
    # this (yet) for better safety: this is handled by @dont_look_inside
    # in call_varargs.
    @jit.unroll_safe
    def _call(self, funcaddr, args_w):
        space = self.space
        cif_descr = self.cif_descr   # 'self' should have been promoted here
        size = cif_descr.exchange_size
        mustfree_max_plus_1 = 0
        keepalives = [llstr(None)] * len(args_w)    # llstrings
        buffer = lltype.malloc(rffi.CCHARP.TO, size, flavor='raw')
        try:
            for i in range(len(args_w)):
                data = rffi.ptradd(buffer, cif_descr.exchange_args[i])
                w_obj = args_w[i]
                argtype = self.fargs[i]
                if argtype.convert_argument_from_object(data, w_obj,
                                                        keepalives, i):
                    # argtype is a pointer type, and w_obj a list/tuple/str
                    mustfree_max_plus_1 = i + 1

            jit_libffi.jit_ffi_call(cif_descr,
                                    rffi.cast(rffi.VOIDP, funcaddr),
                                    buffer)

            resultdata = rffi.ptradd(buffer, cif_descr.exchange_result)
            w_res = self.ctitem.copy_and_convert_to_object(resultdata)
        finally:
            for i in range(mustfree_max_plus_1):
                argtype = self.fargs[i]
                if isinstance(argtype, W_CTypePointer):
                    data = rffi.ptradd(buffer, cif_descr.exchange_args[i])
                    flag = get_mustfree_flag(data)
                    raw_cdata = rffi.cast(rffi.CCHARPP, data)[0]
                    if flag == 1:
                        lltype.free(raw_cdata, flavor='raw')
                    elif flag >= 4:
                        llobj = keepalives[i]
                        assert llobj     # not NULL
                        rffi.free_nonmovingbuffer_ll(raw_cdata,
                                                     llobj, chr(flag))
            lltype.free(buffer, flavor='raw')
            keepalive_until_here(args_w)
        return w_res

def get_mustfree_flag(data):
    return ord(rffi.ptradd(data, -1)[0])

def set_mustfree_flag(data, flag):
    rffi.ptradd(data, -1)[0] = chr(flag)

# ____________________________________________________________


# ----------
# We attach to the classes small methods that return a 'ffi_type'

def _notimplemented_ffi_type(self, is_result_type, extra=''):
    if is_result_type:
        place = "return value"
    else:
        place = "argument"
    raise oefmt(self.space.w_NotImplementedError,
                "ctype '%s' (size %d) not supported as %s%s",
                self.name, self.size, place, extra)

def _missing_ffi_type(self, cifbuilder, is_result_type):
    if self.size < 0:
        raise oefmt(self.space.w_TypeError,
                    "ctype '%s' has incomplete type", self.name)
    raise _notimplemented_ffi_type(self, is_result_type)

def _struct_ffi_type(self, cifbuilder, is_result_type):
    if self.size >= 0:
        return cifbuilder.fb_struct_ffi_type(self, is_result_type)
    return _missing_ffi_type(self, cifbuilder, is_result_type)

def _union_ffi_type(self, cifbuilder, is_result_type):
    if self.size >= 0:   # only for a better error message
        return cifbuilder.fb_union_ffi_type(self, is_result_type)
    return _missing_ffi_type(self, cifbuilder, is_result_type)

def _primsigned_ffi_type(self, cifbuilder, is_result_type):
    size = self.size
    if   size == 1: return clibffi.ffi_type_sint8
    elif size == 2: return clibffi.ffi_type_sint16
    elif size == 4: return clibffi.ffi_type_sint32
    elif size == 8: return clibffi.ffi_type_sint64
    return _missing_ffi_type(self, cifbuilder, is_result_type)

def _primunsigned_ffi_type(self, cifbuilder, is_result_type):
    size = self.size
    if   size == 1: return clibffi.ffi_type_uint8
    elif size == 2: return clibffi.ffi_type_uint16
    elif size == 4: return clibffi.ffi_type_uint32
    elif size == 8: return clibffi.ffi_type_uint64
    return _missing_ffi_type(self, cifbuilder, is_result_type)

def _primfloat_ffi_type(self, cifbuilder, is_result_type):
    size = self.size
    if   size == 4: return clibffi.ffi_type_float
    elif size == 8: return clibffi.ffi_type_double
    return _missing_ffi_type(self, cifbuilder, is_result_type)

def _primlongdouble_ffi_type(self, cifbuilder, is_result_type):
    return clibffi.ffi_type_longdouble

def _primcomplex_ffi_type(self, cifbuilder, is_result_type):
    raise _notimplemented_ffi_type(self, is_result_type,
        extra = " (the support for complex types inside libffi "
                "is mostly missing at this point, so CFFI only "
                "supports complex types as arguments or return "
                "value in API-mode functions)")

def _ptr_ffi_type(self, cifbuilder, is_result_type):
    return clibffi.ffi_type_pointer

def _void_ffi_type(self, cifbuilder, is_result_type):
    if is_result_type:
        return clibffi.ffi_type_void
    return _missing_ffi_type(self, cifbuilder, is_result_type)

W_CType._get_ffi_type                       = _missing_ffi_type
W_CTypeStruct._get_ffi_type                 = _struct_ffi_type
W_CTypeUnion._get_ffi_type                  = _union_ffi_type
W_CTypePrimitiveSigned._get_ffi_type        = _primsigned_ffi_type
W_CTypePrimitiveCharOrUniChar._get_ffi_type = _primunsigned_ffi_type
W_CTypePrimitiveUnsigned._get_ffi_type      = _primunsigned_ffi_type
W_CTypePrimitiveFloat._get_ffi_type         = _primfloat_ffi_type
W_CTypePrimitiveLongDouble._get_ffi_type    = _primlongdouble_ffi_type
W_CTypePrimitiveComplex._get_ffi_type       = _primcomplex_ffi_type
W_CTypePtrBase._get_ffi_type                = _ptr_ffi_type
W_CTypeVoid._get_ffi_type                   = _void_ffi_type
# ----------


_SUPPORTED_IN_API_MODE = (
        " are only supported as %s if the function is "
        "'API mode' and non-variadic (i.e. declared inside ffibuilder"
        ".cdef()+ffibuilder.set_source() and not taking a final '...' "
        "argument)")

class CifDescrBuilder(object):
    rawmem = lltype.nullptr(rffi.CCHARP.TO)

    def __init__(self, fargs, fresult, fabi):
        self.fargs = fargs
        self.fresult = fresult
        self.fabi = fabi

    def fb_alloc(self, size):
        size = llmemory.raw_malloc_usage(size)
        if not self.bufferp:
            self.nb_bytes += size
            return lltype.nullptr(rffi.CCHARP.TO)
        else:
            result = self.bufferp
            self.bufferp = rffi.ptradd(result, size)
            return result

    def fb_fill_type(self, ctype, is_result_type):
        return ctype._get_ffi_type(self, is_result_type)

    def fb_unsupported(self, ctype, is_result_type, detail):
        place = "return value" if is_result_type else "argument"
        raise oefmt(self.space.w_NotImplementedError,
            "ctype '%s' not supported as %s.  %s.  "
            "Such structs" + _SUPPORTED_IN_API_MODE,
            ctype.name, place, detail, place)

    def fb_union_ffi_type(self, ctype, is_result_type=False):
        place = "return value" if is_result_type else "argument"
        raise oefmt(self.space.w_NotImplementedError,
            "ctype '%s' not supported as %s by libffi.  "
            "Unions" + _SUPPORTED_IN_API_MODE,
            ctype.name, place, place)

    def fb_struct_ffi_type(self, ctype, is_result_type=False):
        # We can't pass a struct that was completed by verify().
        # Issue: assume verify() is given "struct { long b; ...; }".
        # Then it will complete it in the same way whether it is actually
        # "struct { long a, b; }" or "struct { double a; long b; }".
        # But on 64-bit UNIX, these two structs are passed by value
        # differently: e.g. on x86-64, "b" ends up in register "rsi" in
        # the first case and "rdi" in the second case.
        #
        # Another reason for 'custom_field_pos' would be anonymous
        # nested structures: we lost the information about having it
        # here, so better safe (and forbid it) than sorry (and maybe
        # crash).  Note: it seems we only get in this case with
        # ffi.verify().
        space = self.space
        ctype.force_lazy_struct()
        if ctype._custom_field_pos:
            # these NotImplementedErrors may be caught and ignored until
            # a real call is made to a function of this type
            raise self.fb_unsupported(ctype, is_result_type,
                "It is a struct declared with \"...;\", but the C "
                "calling convention may depend on the missing fields; "
                "or, it contains anonymous struct/unions")
        # Another reason: __attribute__((packed)) is not supported by libffi.
        if ctype._with_packed_change:
            raise self.fb_unsupported(ctype, is_result_type,
                "It is a 'packed' structure, with a different layout than "
                "expected by libffi")

        # walk the fields, expanding arrays into repetitions; first,
        # only count how many flattened fields there are
        nflat = 0
        for i, cf in enumerate(ctype._fields_list):
            if cf.is_bitfield():
                raise self.fb_unsupported(ctype, is_result_type,
                    "It is a struct with bit fields, which libffi does not "
                    "support")
            flat = 1
            ct = cf.ctype
            while isinstance(ct, ctypearray.W_CTypeArray):
                flat *= ct.length
                ct = ct.ctitem
            if flat <= 0:
                raise self.fb_unsupported(ctype, is_result_type,
                    "It is a struct with a zero-length array, which libffi "
                    "does not support")
            nflat += flat

        # allocate an array of (nflat + 1) ffi_types
        elements = self.fb_alloc(rffi.sizeof(FFI_TYPE_P) * (nflat + 1))
        elements = rffi.cast(FFI_TYPE_PP, elements)

        # fill it with the ffi types of the fields
        nflat = 0
        for i, cf in enumerate(ctype._fields_list):
            flat = 1
            ct = cf.ctype
            while isinstance(ct, ctypearray.W_CTypeArray):
                flat *= ct.length
                ct = ct.ctitem
            ffi_subtype = self.fb_fill_type(ct, False)
            if elements:
                for j in range(flat):
                    elements[nflat] = ffi_subtype
                    nflat += 1

        # zero-terminate the array
        if elements:
            elements[nflat] = lltype.nullptr(FFI_TYPE_P.TO)

        # allocate and fill an ffi_type for the struct itself
        ffistruct = self.fb_alloc(rffi.sizeof(FFI_TYPE))
        ffistruct = rffi.cast(FFI_TYPE_P, ffistruct)
        if ffistruct:
            rffi.setintfield(ffistruct, 'c_size', ctype.size)
            rffi.setintfield(ffistruct, 'c_alignment', ctype.alignof())
            rffi.setintfield(ffistruct, 'c_type', clibffi.FFI_TYPE_STRUCT)
            ffistruct.c_elements = elements

        return ffistruct

    def fb_build(self):
        # Build a CIF_DESCRIPTION.  Actually this computes the size and
        # allocates a larger amount of data.  It starts with a
        # CIF_DESCRIPTION and continues with data needed for the CIF:
        #
        #  - the argument types, as an array of 'ffi_type *'.
        #
        #  - optionally, the result's and the arguments' ffi type data
        #    (this is used only for 'struct' ffi types; in other cases the
        #    'ffi_type *' just points to static data like 'ffi_type_sint32').
        #
        nargs = len(self.fargs)

        # start with a cif_description (cif and exchange_* fields)
        self.fb_alloc(llmemory.sizeof(CIF_DESCRIPTION, nargs))

        # next comes an array of 'ffi_type*', one per argument
        atypes = self.fb_alloc(rffi.sizeof(FFI_TYPE_P) * nargs)
        self.atypes = rffi.cast(FFI_TYPE_PP, atypes)

        # next comes the result type data
        self.rtype = self.fb_fill_type(self.fresult, True)

        # next comes each argument's type data
        for i, farg in enumerate(self.fargs):
            atype = self.fb_fill_type(farg, False)
            if self.atypes:
                self.atypes[i] = atype

    def align_arg(self, n):
        return (n + 7) & ~7

    def align_to(self, n, type):
        a = rffi.getintfield(type, 'c_alignment') - 1
        return (n + a) & ~a

    def fb_build_exchange(self, cif_descr):
        nargs = len(self.fargs)

        # first, enough room for an array of 'nargs' pointers
        exchange_offset = rffi.sizeof(rffi.CCHARP) * nargs

        # then enough room for the result --- which means at least
        # sizeof(ffi_arg), according to the ffi docs, but we also
        # align according to the result type, for cffi issue #531
        exchange_offset = self.align_to(exchange_offset, self.rtype)
        exchange_offset = self.align_arg(exchange_offset)
        cif_descr.exchange_result = exchange_offset
        exchange_offset += max(rffi.getintfield(self.rtype, 'c_size'),
                               SIZE_OF_FFI_ARG)

        # loop over args
        for i, farg in enumerate(self.fargs):
            if isinstance(farg, W_CTypePointer):
                exchange_offset += 1   # for the "must free" flag
            atype = self.atypes[i]
            exchange_offset = self.align_to(exchange_offset, atype)
            exchange_offset = self.align_arg(exchange_offset)
            cif_descr.exchange_args[i] = exchange_offset
            exchange_offset += rffi.getintfield(atype, 'c_size')

        # store the exchange data size
        # we also align it to the next multiple of 8, in an attempt to
        # work around bugs(?) of libffi (see cffi issue #241)
        cif_descr.exchange_size = self.align_arg(exchange_offset)

    def fb_extra_fields(self, cif_descr):
        cif_descr.abi = self.fabi
        cif_descr.nargs = len(self.fargs)
        cif_descr.rtype = self.rtype
        cif_descr.atypes = self.atypes

    @jit.dont_look_inside
    def rawallocate(self, ctypefunc):
        space = ctypefunc.space
        self.space = space

        # compute the total size needed in the CIF_DESCRIPTION buffer
        self.nb_bytes = 0
        self.bufferp = lltype.nullptr(rffi.CCHARP.TO)
        self.fb_build()

        # allocate the buffer
        if we_are_translated():
            rawmem = lltype.malloc(rffi.CCHARP.TO, self.nb_bytes,
                                   flavor='raw')
            rawmem = rffi.cast(CIF_DESCRIPTION_P, rawmem)
        else:
            # gross overestimation of the length below, but too bad
            rawmem = lltype.malloc(CIF_DESCRIPTION_P.TO, self.nb_bytes,
                                   flavor='raw')

        # the buffer is automatically managed from the W_CTypeFunc instance
        ctypefunc.cif_descr = rawmem

        # call again fb_build() to really build the libffi data structures
        self.bufferp = rffi.cast(rffi.CCHARP, rawmem)
        self.fb_build()
        assert self.bufferp == rffi.ptradd(rffi.cast(rffi.CCHARP, rawmem),
                                           self.nb_bytes)

        # fill in the 'exchange_*' fields
        self.fb_build_exchange(rawmem)

        # fill in the extra fields
        self.fb_extra_fields(rawmem)

        # call libffi's ffi_prep_cif() function
        res = jit_libffi.jit_ffi_prep_cif(rawmem)
        if res != clibffi.FFI_OK:
            raise oefmt(space.w_SystemError,
                        "libffi failed to build this function type")



from pypy.interpreter.astcompiler import ast
class TestAstToObject:
    def test_types(self, space):
        assert space.issubtype_w(
                ast.get(space).w_Module, ast.get(space).w_mod)
                                  
    def test_num(self, space):
        value = space.wrap(42)
        node = ast.Num(value, lineno=1, col_offset=1)
        w_node = node.to_object(space)
        assert space.is_w(space.getattr(w_node, space.wrap("n")), value)

    def test_expr(self, space):
        value = space.wrap(42)
        node = ast.Num(value, lineno=1, col_offset=1)
        expr = ast.Expr(node, lineno=1, col_offset=1)
        w_node = expr.to_object(space)
        # node.value.n
        assert space.is_w(space.getattr(space.getattr(w_node, space.wrap("value")),
                          space.wrap("n")), value)

    def test_operation(self, space):
        val1 = ast.Num(space.wrap(1), lineno=1, col_offset=1)
        val2 = ast.Num(space.wrap(2), lineno=1, col_offset=1)
        node = ast.BinOp(left=val1, right=val2, op=ast.Add,
                         lineno=1, col_offset=1)
        w_node = node.to_object(space)
        w_op = space.getattr(w_node, space.wrap("op"))
        assert space.isinstance_w(w_op, ast.get(space).w_operator)

    def test_from_object(self, space):
        value = space.wrap(42)
        w_node = space.call_function(ast.get(space).w_Num)
        space.setattr(w_node, space.wrap('n'), value)
        space.setattr(w_node, space.wrap('lineno'), space.wrap(1))
        space.setattr(w_node, space.wrap('col_offset'), space.wrap(1))
        node = ast.Num.from_object(space, w_node)
        assert space.is_w(node.n, value)

    def test_fields(self, space):
        w_fields = space.getattr(ast.get(space).w_FunctionDef,
                                 space.wrap("_fields"))
        assert space.eq_w(w_fields, space.wrap(
            ('name', 'args', 'body', 'decorator_list')))
        w_fields = space.getattr(ast.get(space).w_arguments,
                                 space.wrap("_fields"))
        assert space.eq_w(w_fields, space.wrap(
            ('args', 'vararg', 'kwarg', 'defaults')))
        
    def test_attributes(self, space):
        w_attrs = space.getattr(ast.get(space).w_FunctionDef,
                                space.wrap("_attributes"))
        assert space.eq_w(w_attrs, space.wrap(('lineno', 'col_offset')))
        



import py
from rpython.rtyper.lltypesystem import lltype
from rpython.rtyper.lltypesystem.lloperation import llop
from rpython.rlib import rgc

from rpython.memory.test import test_minimark_gc

class TestIncrementalMiniMarkGC(test_minimark_gc.TestMiniMarkGC):
    from rpython.memory.gc.incminimark import IncrementalMiniMarkGC as GCClass
    WREF_IS_INVALID_BEFORE_DEL_IS_CALLED = True

    def test_weakref_not_in_stack(self):
        import weakref
        class A(object):
            pass
        class B(object):
            def __init__(self, next):
                self.next = next
        def g():
            a = A()
            a.x = 5
            wr = weakref.ref(a)
            llop.gc__collect(lltype.Void)   # make everything old
            assert wr() is not None
            assert a.x == 5
            return wr
        def f():
            ref = g()
            llop.gc__collect(lltype.Void, 1)    # start a major cycle
            # at this point the stack is scanned, and the weakref points
            # to an object not found, but still reachable:
            b = ref()
            llop.debug_print(lltype.Void, b)
            assert b is not None
            llop.gc__collect(lltype.Void)   # finish the major cycle
            # assert does not crash, because 'b' is still kept alive
            b.x = 42
            return ref() is b
        res = self.interpret(f, [])
        assert res == True

    def test_pin_weakref_not_implemented(self):
        import weakref
        class A:
            pass
        def f():
            a = A()
            ref = weakref.ref(a)
            assert not rgc.pin(ref)
        self.interpret(f, [])

    def test_pin_finalizer_not_implemented(self):
        import weakref
        class A:
            def __del__(self):
                pass
        class B:
            def __del__(self):
                foo.bar += 1
        class Foo:
            bar = 0
        foo = Foo()
        def f():
            a = A()
            b = B()
            assert not rgc.pin(a)
            assert not rgc.pin(b)
        self.interpret(f, [])

    def test_weakref_to_pinned(self):
        py.test.skip("weakref to pinned object: not supported")
        import weakref
        from rpython.rlib import rgc
        class A(object):
            pass
        def g():
            a = A()
            assert rgc.pin(a)
            a.x = 100
            wr = weakref.ref(a)
            llop.gc__collect(lltype.Void)
            assert wr() is not None
            assert a.x == 100
            return wr
        def f():
            ref = g()
            llop.gc__collect(lltype.Void, 1)
            b = ref()
            assert b is not None
            b.x = 101
            return ref() is b
        res = self.interpret(f, [])
        assert res == True




from rpython.jit.metainterp.test.test_tracelimit import TraceLimitTests
from rpython.jit.backend.x86.test.test_basic import Jit386Mixin

class TestTraceLimit(Jit386Mixin, TraceLimitTests):
    # for the individual tests see
    # ====> ../../../metainterp/test/test_tracelimit.py
    pass



import sys, py
import random
from rpython.jit.backend.x86 import rx86
from rpython.jit.backend.x86.test import test_rx86_32_auto_encoding

if sys.maxint <= 2**32:
    py.test.skip("skipping this test on x86-32")


class TestRx86_64(test_rx86_32_auto_encoding.TestRx86_32):
    WORD = 8
    TESTDIR = 'rx86_64'
    X86_CodeBuilder = rx86.X86_64_CodeBuilder
    REGNAMES = ['%rax', '%rcx', '%rdx', '%rbx', '%rsp', '%rbp', '%rsi', '%rdi',
                '%r8', '%r9', '%r10', '%r11', '%r12', '%r13', '%r14', '%r15']
    REGNAMES8 = ['%al', '%cl', '%dl', '%bl', '%spl', '%bpl', '%sil', '%dil',
                '%r8b', '%r9b', '%r10b', '%r11b',
                 '%r12b', '%r13b', '%r14b', '%r15b']
    REGS = range(16)
    REGS8 = [i|rx86.BYTE_REG_FLAG for i in range(16)]
    NONSPECREGS = [rx86.R.eax, rx86.R.ecx, rx86.R.edx, rx86.R.ebx,
                   rx86.R.esi, rx86.R.edi,
                   rx86.R.r8,  rx86.R.r9,  rx86.R.r10, rx86.R.r11,
                   rx86.R.r12, rx86.R.r13, rx86.R.r14, rx86.R.r15]
    accept_unnecessary_prefix = '\x40'

    def should_skip_instruction(self, instrname, argmodes):
        return (
                super(TestRx86_64, self).should_skip_instruction(instrname, argmodes) or
                # Not testing FSTP on 64-bit for now
                (instrname == 'FSTP') or
                # the test suite uses 64 bit registers instead of 32 bit...
                # it is tested in the 32 bit test!
                (instrname == 'PEXTRD') or
                (instrname == 'PINSRD')
        )

    def array_tests(self):
        # reduce a little bit -- we spend too long in these tests
        lst = super(TestRx86_64, self).array_tests()
        random.shuffle(lst)
        return lst[:int(len(lst) * 0.2)]

    def imm64_tests(self):
        v = [-0x80000001, 0x80000000,
             -0x8000000000000000, 0x7FFFFFFFFFFFFFFF]
        for i in range(test_rx86_32_auto_encoding.COUNT1):
            x = ((random.randrange(-32768,32768)<<48) |
                 (random.randrange(0,65536)<<32) |
                 (random.randrange(0,65536)<<16) |
                 (random.randrange(0,65536)<<0))
            v.append(x)
        return v + super(TestRx86_64, self).imm32_tests()

    def test_extra_MOV_ri64(self):
        self.imm32_tests = self.imm64_tests      # patch on 'self'
        self.complete_test('MOV_ri')

    def rip_relative_tests(self):
        return [-0x80000000, 0x7FFFFFFF, 128, 256, -129, -255, 0, 127]

    def get_all_tests(self):
        d = super(TestRx86_64, self).get_all_tests()
        d['p'] = self.rip_relative_tests
        return d

    def assembler_operand_rip_relative(self, value):
        return '%d(%%rip)' % value

    def get_all_assembler_operands(self):
        d = super(TestRx86_64, self).get_all_assembler_operands()
        d['p'] = self.assembler_operand_rip_relative
        return d



import py
from rpython.rlib.objectmodel import r_dict
from rpython.rtyper.test.tool import BaseRtypingTest

class TestRconstantdict(BaseRtypingTest):

    def test_constant_int_dict(self):
        d = {1: 2, 2: 3, 3: 4}
        def func(i):
            return d[i]
        res = self.interpret(func, [3])
        assert res == 4

    def test_constantdict_contains(self):
        d = {1: True, 4: True, 16: True}
        def func(i):
            return i in d
        res = self.interpret(func, [15])
        assert res is False
        res = self.interpret(func, [4])
        assert res is True

    def test_constantdict_get(self):
        d = {1: -11, 4: -44, 16: -66}
        def func(i, j):
            return d.get(i, j)
        res = self.interpret(func, [15, 62])
        assert res == 62
        res = self.interpret(func, [4, 25])
        assert res == -44

    def test_unichar_dict(self):
        d = {u'a': 5, u'b': 123, u'?': 321}
        def func(i):
            return d[unichr(i)]
        res = self.interpret(func, [97])
        assert res == 5
        res = self.interpret(func, [98])
        assert res == 123
        res = self.interpret(func, [63])
        assert res == 321

    def test_constant_r_dict(self):
        def strange_key_eq(key1, key2):
            return key1[0] == key2[0]   # only the 1st character is relevant
        def strange_key_hash(key):
            return ord(key[0])

        d = r_dict(strange_key_eq, strange_key_hash)
        d['hello'] = 42
        d['world'] = 43
        for x in range(65, 91):
            d[chr(x)] = x*x
        def func(i):
            return d[chr(i)]
        res = self.interpret(func, [ord('h')])
        assert res == 42
        res = self.interpret(func, [ord('w')])
        assert res == 43
        for x in range(65, 91):
            res = self.interpret(func, [x])
            assert res == x*x



# -*- coding: utf-8 -*-
import sys
from pypy.module.pypyjit.test_pypy_c.test_00_model import BaseTestPyPyC

# XXX review the <Call> descrs to replace some EF=5 with EF=4 (elidable)


class TestString(BaseTestPyPyC):
    def test_lookup_default_encoding(self):
        def main(n):
            import string
            i = 0
            letters = string.letters
            uletters = unicode(string.letters)
            while i < n:
                i += letters[i % len(letters)] == uletters[i % len(letters)]
            return i

        log = self.run(main, [300], import_site=True)
        assert log.result == 300
        loop, = log.loops_by_filename(self.filepath)
        assert loop.match("""
            i14 = int_lt(i6, i9)
            guard_true(i14, descr=...)
            guard_not_invalidated(descr=...)
            i16 = int_eq(i6, %d)
            i83 = call_i(ConstClass(ll_int_py_mod__Signed_Signed), i6, i10, descr=<Calli . ii EF=0 OS=14>)
            i22 = int_ge(i83, i10)
            guard_false(i22, descr=...)
            i89 = strgetitem(p55, i83)
            i24 = int_ge(i83, i12)
            guard_false(i24, descr=...)
            i93 = int_add(i83, 1)
            i94 = int_gt(i93, i12)
            guard_false(i94, descr=...)
            p96 = newstr(1)
            strsetitem(p96, 0, i89)
            i98 = call_i(ConstClass(first_non_ascii_char), p96, descr=<Calli . r EF=4>)
            guard_no_exception(descr=...)
            i100 = int_lt(i98, 0)
            guard_true(i100, descr=...)
            i102 = call_i(ConstClass(_ll_4_str_eq_slice_char__rpy_stringPtr_Signed_Signed_Char), p13, i83, 1, i89, descr=<Calli . riii EF=0 OS=27>)
            guard_true(i102, descr=...)
            i104 = int_add(i6, 1)
            --TICK--
            jump(..., descr=...)
        """ % (-sys.maxint-1,))

    def test_long(self):
        def main(n):
            import string
            i = 1
            while i < n:
                i += int(long(string.digits[i % len(string.digits)], 16))
            return i

        log = self.run(main, [1100], import_site=True)
        assert log.result == main(1100)
        loop, = log.loops_by_filename(self.filepath)
        if sys.maxint > 2**32:
            args = (63, -3689348814741910323, 3)
        else:
            args = (31, -858993459, 3)
        assert loop.match("""
            i11 = int_lt(i6, i7)
            guard_true(i11, descr=...)
            guard_not_invalidated(descr=...)
            i13 = int_eq(i6, %d)         # value provided below

            # "mod 10" block:
            i79 = int_rshift(i6, %d)
            i80 = int_xor(i6, i79)
            i82 = uint_mul_high(i80, %d)
            i84 = uint_rshift(i82, %d)
            i85 = int_xor(i84, i79)
            i87 = int_mul(i85, 10)
            i19 = int_sub(i6, i87)

            i23 = strgetitem(p10, i19)
            p25 = newstr(1)
            strsetitem(p25, 0, i23)
            p93 = call_r(ConstClass(fromstr), p25, 16, 0, descr=<Callr . rii EF=4>)
            guard_no_exception(descr=...)
            i94 = call_i(ConstClass(rbigint.toint), p93, descr=<Calli . r EF=4>)
            guard_no_exception(descr=...)
            i95 = int_add_ovf(i6, i94)
            guard_no_overflow(descr=...)
            --TICK--
            jump(..., descr=...)
        """ % ((-sys.maxint-1,)+args))

    def test_str_mod(self):
        def main(n):
            s = 0
            while n > 0:
                s += len('%d %d' % (n, n))
                n -= 1
            return s

        log = self.run(main, [1000])
        assert log.result == main(1000)
        loop, = log.loops_by_filename(self.filepath)
        assert loop.match("""
            i79 = int_gt(i74, 0)
            guard_true(i79, descr=...)
            guard_not_invalidated(descr=...)
            p80 = call_r(ConstClass(ll_int2dec__Signed), i74, descr=<Callr . i EF=3>)
            guard_no_exception(descr=...)
            i85 = strlen(p80)
            p86 = new(descr=<SizeDescr .+>)
            p88 = newstr(23)
            {{{
            setfield_gc(p86, 0, descr=<FieldS stringbuilder.current_pos .+>)
            setfield_gc(p86, p88, descr=<FieldP stringbuilder.current_buf .+>)
            setfield_gc(p86, 23, descr=<FieldS stringbuilder.current_end .+>)
            setfield_gc(p86, 23, descr=<FieldS stringbuilder.total_size .+>)
            }}}
            call_n(ConstClass(ll_append_res0__stringbuilderPtr_rpy_stringPtr), p86, p80, descr=<Callv 0 rr EF=5>)
            guard_no_exception(descr=...)
            i89 = getfield_gc_i(p86, descr=<FieldS stringbuilder.current_pos .+>)
            i90 = getfield_gc_i(p86, descr=<FieldS stringbuilder.current_end .+>)
            i91 = int_eq(i89, i90)
            cond_call(i91, ConstClass(ll_grow_by__stringbuilderPtr_Signed), p86, 1, descr=<Callv 0 ri EF=5>)
            guard_no_exception(descr=...)
            i92 = getfield_gc_i(p86, descr=<FieldS stringbuilder.current_pos .+>)
            i93 = int_add(i92, 1)
            p94 = getfield_gc_r(p86, descr=<FieldP stringbuilder.current_buf .+>)
            strsetitem(p94, i92, 32)
            setfield_gc(p86, i93, descr=<FieldS stringbuilder.current_pos .+>)
            call_n(ConstClass(ll_append_res0__stringbuilderPtr_rpy_stringPtr), p86, p80, descr=<Callv 0 rr EF=5>)
            guard_no_exception(descr=...)
            p95 = call_r(..., descr=<Callr . r EF=5>)     # ll_build
            guard_no_exception(descr=...)
            i96 = strlen(p95)
            i97 = int_add_ovf(i71, i96)
            guard_no_overflow(descr=...)
            i98 = int_sub(i74, 1)
            --TICK--
            jump(..., descr=...)
        """)

    def test_remove_duplicate_method_calls_bytes(self):
        def main(n):
            lst = []
            for i in range(n):
                s = b'Hello %d' % i
                t = s.lower()   # ID: callone
                u = s.lower()   # ID: calltwo
                lst.append(t)
                lst.append(u)
            return len(','.join(lst))
        log = self.run(main, [1000])
        assert log.result == main(1000)
        loops = log.loops_by_filename(self.filepath)
        loop, = loops
        assert loop.match_by_id('callone', '''
            p114 = call_r(ConstClass(ll_lower__rpy_stringPtr), p113, descr=<Callr . r EF=3>)
            guard_no_exception(descr=...)
            ''')
        assert loop.match_by_id('calltwo', '')    # nothing

    def test_remove_duplicate_method_calls_unicode(self):
        def main(n):
            lst = []
            for i in range(n):
                s = u'H\xe4llo %d' % i
                t = s.lower()   # ID: callone
                u = s.lower()   # ID: calltwo
                lst.append(t)
                lst.append(u)
            return len(','.join(lst))
        log = self.run(main, [1000])
        assert log.result == main(1000)
        loops = log.loops_by_filename(self.filepath)
        loop, = loops
        assert loop.match_by_id('callone', '''
            i136 = strlen(p131)
            i137 = int_eq(i135, i136)
            guard_false(i137, descr=...)
            p139 = call_r(ConstClass(_lower_unicode), p131, descr=...)
            guard_no_exception(descr=...)
            guard_nonnull(p139, descr=...)
            ''')
        assert loop.match_by_id('calltwo', '')    # nothing

    def test_move_method_call_out_of_loop(self):
        def main(n):
            lst = []
            s = 'Hello %d' % n
            for i in range(n):
                t = s.lower()   # ID: callone
                lst.append(t)
            return len(','.join(lst))
        log = self.run(main, [1000])
        assert log.result == main(1000)
        loops = log.loops_by_filename(self.filepath)
        loop, = loops
        assert loop.match_by_id('callone', '')    # nothing

    def test_lookup_codec(self):
        log = self.run("""
        import codecs

        def main(n):
            for i in xrange(n):
                codecs.lookup('utf8')
            return i
        """, [1000])
        loop, = log.loops_by_filename(self.filepath)
        assert loop.match("""
        i45 = int_lt(i43, i26)
        guard_true(i45, descr=...)
        i46 = int_add(i43, 1)
        setfield_gc(p15, i46, descr=<FieldS pypy.module.__builtin__.functional.W_XRangeIterator.inst_current 8>)
        guard_not_invalidated(descr=...)
        --TICK--
        jump(..., descr=...)
        """)

    def test_decode_ascii(self):
        log = self.run("""
        def main(n):
            for i in xrange(n):
                unicode(str(i))
            return i
        """, [1000])
        loop, = log.loops_by_filename(self.filepath)
        assert loop.match("""
        i49 = int_lt(i47, i24)
        guard_true(i49, descr=...)
        i50 = int_add(i47, 1)
        setfield_gc(p15, i50, descr=<FieldS pypy.module.__builtin__.functional.W_XRangeIterator.inst_current 8>)
        guard_not_invalidated(descr=...)
        p80 = call_r(ConstClass(ll_str__IntegerR_SignedConst_Signed), i47, descr=<Callr . i EF=3>)
        guard_no_exception(descr=...)
        i51 = call_i(ConstClass(first_non_ascii_char), p80, descr=<Calli . r EF=4>)
        guard_no_exception(descr=...)
        i52 = int_lt(i51, 0)
        guard_true(i52, descr=...)
        i53 = strlen(p80)
        --TICK--
        jump(..., descr=...)
        """)

    def test_unicode_indexing_makes_no_bridges(self):
        log = self.run("""
        b = b"b'aaaaa\xc3\xa4\xf0\x9f\x91\xa9\xe2\x80\x8d\xf0\x9f\x91\xa9\xe2\x80\x8d\xf0\x9f\x91\xa7\xe2\x80\x8d\xf0\x9f\x91\xa6'"
        u = b.decode("utf-8") * 1000
        def main():
            for j in range(10):
                for i in range(len(u)):
                    u[i] # ID: index0
        """, [])
        ops = log.loops[0].ops_by_id("index0")
        for op in ops:
            assert op.bridge is None

    def test_unicode_indexing_small_constant_indices(self):
        log = self.run("""
        l = [u"abä", u"cdä", u"äü", u"éé", u"–—¿"] * 1000
        def main(n):
            global s
            for u in l:
                s = u[0] + u[1] + u[-1] # ID: index
                len(u)
            return len(s)
        """, [1000])
        loop, = log.loops_by_filename(self.filepath)
        assert loop.match_by_id('index', '''
            i77 = getfield_gc_i(p73, descr=<FieldS pypy.objspace.std.unicodeobject.W_UnicodeObject.inst__length .*>)
            p78 = getfield_gc_r(p73, descr=<FieldP pypy.objspace.std.unicodeobject.W_UnicodeObject.inst__utf8 .* pure>)
            i79 = strlen(p78)
            i80 = int_eq(i77, i79)
            guard_false(i80, descr=...) # check not ascii
            i82 = int_ge(0, i77)
            guard_false(i82, descr=...)
            i85 = call_i(ConstClass(next_codepoint_pos_dont_look_inside), p78, 0, descr=...)
            i86 = int_gt(i85, i79)
            guard_false(i86, descr=...)
            i88 = int_ge(1, i77)
            guard_false(i88, descr=...)
            i90 = call_i(ConstClass(next_codepoint_pos_dont_look_inside), p78, i85, descr=...)
            i91 = int_gt(i90, i79)
            guard_false(i91, descr=...)
            i92 = int_sub(i90, i85)
            i94 = int_add(-1, i77)
            i96 = call_i(ConstClass(prev_codepoint_pos_dont_look_inside), p78, i79, descr=...)
            i97 = int_sub(i79, i96)
            guard_not_invalidated(descr=...)
        ''')

    def test_unicode_slicing_small_constant_indices(self):
        log = self.run("""
        def main(n):
            b = b'ab\xc3\xa4\xf0\x9f\x91\xa9\xe2\x80\x8d\xf0\x9f\x91\xa9\xe2\x80\x8d\xf0\x9f\x91\xa7\xe2\x80\x8d\xf0\x9f\x91\xa6'
            u = b.decode("utf-8") * 1000
            global s
            count = 0
            while u:
                u = u[1:] # ID: index
                count += 1
            return count
        """, [1000])
        loop, = log.loops_by_filename(self.filepath)
        assert loop.match_by_id('index', '''
            i51 = int_eq(1, i38)
            guard_false(i51, descr=...)
            i52 = strlen(p47)
            i53 = int_eq(i38, i52)
            guard_false(i53, descr=...)
            i56 = call_i(ConstClass(next_codepoint_pos_dont_look_inside), p47, 0, descr=...)
            i57 = int_sub(i52, i56)
            i59 = int_sub(i38, 1)
        ''')

    def test_decode_encode(self):
        log = self.run("""
        def main(n):
            global s
            u = b'ab\xc3\xa4\xf0\x9f\x91\xa9\xe2\x80\x8d\xf0\x9f\x91\xa9\xe2\x80\x8d\xf0\x9f\x91\xa7\xe2\x80\x8d\xf0\x9f\x91\xa6'.decode("utf-8")
            count = 0
            for i in range(n):
                b = (u + unicode(i)).encode("utf-8")
                u = b.decode("utf-8") # ID: decode
                count += 1
            return count
        """, [10000])
        loop, = log.loops_by_filename(self.filepath)
        # No call to _check_utf8 is necessary, because the bytes come from
        # W_UnicodeObject.utf8_w.
        assert loop.match_by_id('decode', '''
            i95 = int_ge(i86, 0)
            guard_true(i95, descr=...)
        ''')

    def test_find(self):
        log = self.run("""
        def main(n):
            global s
            b = b'abaaac' * 10000
            count = 0
            start = 0
            while 1:
                start = b.find('a', start)
                if start < 0:
                    break
                b[start] # ID: index
                count += 1
                start += 1
            return count
        """, [10000])
        loop, = log.loops_by_filename(self.filepath)
        # no bounds check, the < 0 is done by the check for end of string.
        # the >= len is not necessary, because find has a record_exact_value
        # that records the fact that this is always true.
        assert loop.match_by_id('index', '''
            i2 = strgetitem(p1, i1)
        ''')

    def test_strip_doesnt_escape_bytes(self):
        log = self.run("""
        def main(n):
            l = [unicode(x).decode("ascii") for x in range(10000)]
            res = 0
            for data in l:
                res += len(data.strip(b'1')) # ID: striparg
            for data in l:
                res += len(data.strip()) # ID: stripnone
            return res
        """, [10000])
        _, loop1, loop2 = log.loops_by_filename(self.filepath)
        opnames = log.opnames(loop1.ops_by_id('striparg'))
        assert "new_with_vtable" not in opnames
        assert "call_may_force_r" not in opnames
        assert opnames.count("call_i") == 2 # _strip_bytes_unboxed_left/right
        opnames = log.opnames(loop2.ops_by_id('stripnone'))
        assert "new_with_vtable" not in opnames
        assert "call_may_force_r" not in opnames
        assert opnames.count("call_i") == 2 # _strip_bytes_unboxed_left/right

    def test_unicode_strip_doesnt_escape_uniobject(self):
        log = self.run("""
        def main(n):
            uni = b'\xc3\xa4'.decode("utf-8")
            l = [unicode(x) + uni + unicode(x) for x in range(10000)]
            res = 0
            for data in l:
                res += len(data.strip(u'1')) # ID: stripnone
            for data in l:
                res += len(data.strip()) # ID: striparg
            return res
        """, [10000])
        _, loop1, loop2 = log.loops_by_filename(self.filepath)
        opnames = log.opnames(loop1.ops_by_id('stripnone'))
        assert "new_with_vtable" not in opnames
        assert "call_may_force_r" not in opnames
        assert "call_r" in opnames # _strip_unboxed
        opnames = log.opnames(loop2.ops_by_id('striparg'))
        assert "new_with_vtable" not in opnames
        assert "call_may_force_r" not in opnames
        assert "call_r" in opnames # _strip_none_unboxed

    def test_unicode_strip_doesnt_escape_uniobject_ascii(self):
        log = self.run("""
        def main(n):
            l = [unicode(x) for x in range(10000)]
            res = 0
            for data in l:
                res += len(data.strip(u'1')) # ID: stripnone
            for data in l:
                res += len(data.strip()) # ID: striparg
            return res
        """, [10000])
        _, loop1, loop2 = log.loops_by_filename(self.filepath)
        opnames = log.opnames(loop1.ops_by_id('stripnone'))
        assert "new_with_vtable" not in opnames
        assert "call_may_force_r" not in opnames
        assert "call_r" not in opnames
        assert opnames.count("call_i") == 2 # _strip_ascii_unboxed_left/right
        opnames = log.opnames(loop2.ops_by_id('striparg'))
        assert "new_with_vtable" not in opnames
        assert "call_may_force_r" not in opnames
        assert "call_r" not in opnames
        assert opnames.count("call_i") == 2 # _strip_none_ascii_unboxed_left/right




import py
from pypy.interpreter.baseobjspace import W_Root, ObjSpace
from pypy.objspace.std.test.test_proxy_internals import AppProxy
from pypy.interpreter.typedef import TypeDef
from pypy.interpreter.gateway import interp2app
from pypy.objspace.std.transparent import register_proxyable
from pypy.conftest import option


class W_Wrapped(W_Root):
    def new(space, w_type):
        return space.wrap(W_Wrapped())

    def name(self, space):
        return space.wrap("wrapped")
    name.unwrap_spec = ['self', ObjSpace]

W_Wrapped.typedef = TypeDef(
    'Wrapped',
    __new__ = interp2app(W_Wrapped.new.im_func),
    __name__ = interp2app(W_Wrapped.name),
)


class AppTestProxyNewtype(AppProxy):
    def setup_class(cls):
        if option.runappdirect:
            py.test.skip("Impossible to run on appdirect")
        AppProxy.setup_class.im_func(cls)
        cls.w_wrapped = cls.space.wrap(W_Wrapped())
        register_proxyable(cls.space, W_Wrapped)

    def test_one(self):
        x = type(self.wrapped)()
        from __pypy__ import tproxy

        def f(name, *args, **kwds):
            return getattr(x, name)(*args, **kwds)

        t = tproxy(type(x), f)
        assert t.__name__ == x.__name__



from rpython.annotator import model
from rpython.annotator.listdef import ListDef
from rpython.annotator.dictdef import DictDef


def none():
    return model.s_None


def impossible():
    return model.s_ImpossibleValue


def float():
    return model.SomeFloat()


def singlefloat():
    return model.SomeSingleFloat()


def longfloat():
    return model.SomeLongFloat()


def int():
    return model.SomeInteger()

def int_nonneg():
    return model.SomeInteger(nonneg=True)

def bool():
    return model.SomeBool()


def unicode():
    return model.SomeUnicodeString()


def unicode0():
    return model.SomeUnicodeString(no_nul=True)


def str(can_be_None=False):
    return model.SomeString(can_be_None=can_be_None)


def bytearray():
    return model.SomeByteArray()


def str0():
    return model.SomeString(no_nul=True)


def char():
    return model.SomeChar()


def ptr(ll_type):
    from rpython.rtyper.lltypesystem.lltype import Ptr
    from rpython.rtyper.llannotation import SomePtr
    return SomePtr(Ptr(ll_type))


def list(element):
    listdef = ListDef(None, element, mutated=True, resized=True)
    return model.SomeList(listdef)


def array(element):
    listdef = ListDef(None, element, mutated=True, resized=False)
    return model.SomeList(listdef)


def dict(keytype, valuetype):
    dictdef = DictDef(None, keytype, valuetype)
    return model.SomeDict(dictdef)


def instance(cls, can_be_None=False):
    return lambda bookkeeper: model.SomeInstance(bookkeeper.getuniqueclassdef(cls), can_be_None=can_be_None)


class SelfTypeMarker(object):
    pass


def self():
    return SelfTypeMarker()


class AnyTypeMarker(object):
    pass


def any():
    return AnyTypeMarker()



from rpython.jit.metainterp.optimizeopt.optimizer import Optimization
from rpython.jit.metainterp.optimizeopt.util import make_dispatcher_method
from rpython.jit.metainterp.resoperation import rop, OpHelpers

class OptSimplify(Optimization):
    def emit(self, op):
        if op.is_guard():
            if self.optimizer.pendingfields is None:
                self.optimizer.pendingfields = []
        return Optimization.emit(self, op)

    def optimize_CALL_PURE_I(self, op):
        opnum = OpHelpers.call_for_descr(op.getdescr())
        newop = self.optimizer.replace_op_with(op, opnum)
        return self.emit(newop)
    optimize_CALL_PURE_R = optimize_CALL_PURE_I
    optimize_CALL_PURE_F = optimize_CALL_PURE_I
    optimize_CALL_PURE_N = optimize_CALL_PURE_I

    def optimize_CALL_LOOPINVARIANT_I(self, op):
        opnum = OpHelpers.call_for_descr(op.getdescr())
        op = op.copy_and_change(opnum)
        return self.emit(op)
    optimize_CALL_LOOPINVARIANT_R = optimize_CALL_LOOPINVARIANT_I
    optimize_CALL_LOOPINVARIANT_F = optimize_CALL_LOOPINVARIANT_I
    optimize_CALL_LOOPINVARIANT_N = optimize_CALL_LOOPINVARIANT_I

    def optimize_VIRTUAL_REF_FINISH(self, op):
        pass

    def optimize_VIRTUAL_REF(self, op):
        newop = self.replace_op_with(op, rop.SAME_AS_R, [op.getarg(0)])
        return self.emit(newop)

    def optimize_QUASIIMMUT_FIELD(self, op):
        # xxx ideally we could also kill the following GUARD_NOT_INVALIDATED
        #     but it's a bit hard to implement robustly if heap.py is also run
        pass

    def optimize_ASSERT_NOT_NONE(self, op):
        pass

    def optimize_RECORD_EXACT_CLASS(self, op):
        pass

    def optimize_RECORD_EXACT_VALUE_R(self, op):
        pass

    def optimize_RECORD_EXACT_VALUE_I(self, op):
        pass

    def optimize_GUARD_FUTURE_CONDITION(self, op):
        self.optimizer.notice_guard_future_condition(op)

dispatch_opt = make_dispatcher_method(OptSimplify, 'optimize_',
                                      default=OptSimplify.emit)
OptSimplify.propagate_forward = dispatch_opt



import os
import py

try:
    import Image
except ImportError as e:
    py.test.skip(str(e))

from pypy.tool.bench.htmlreport import (
        PerfResult, PerfTable, PerfResultCollection,
        PerfResultDelta, Page
)

# Sample performance data with a newline 
testlines = """\
--date 1152625530.0  hacker@canonical.com-20060705122759-0a3481a4647b16dc
 100ms package.module.test_name1

 200ms package.module.test_name2

""".splitlines()

class TestPerfTable:
    """tests to ensure that performance data files are processed properly"""
    
    def setup_method(self, method):
        self.lines = testlines[:]
        self.num_test_ids = 2

    def test_parse(self):
        """read one line and ensure the data is read correctly"""
        lines = self.lines[:2]
        t = PerfTable()
        data = list(t.parse(lines))
        assert len(data) == 1 # check that one object was created
        
        assert isinstance(data[0], PerfResult) # is it really a PerfResult
        pr = data[0]
        expected = PerfResult(
            date=1152625530.0,
            revision_id='hacker@canonical.com-20060705122759-0a3481a4647b16dc',
            elapsed_time=100,
            test_id=('package.module.test_name1'),
        )
        assert pr.test_id == expected.test_id
        assert pr.revision_id == expected.revision_id
        assert pr.elapsed_time == expected.elapsed_time
        assert pr.date == expected.date

    def test_multiline_parse(self):
        """ensure all lines are parsed correctly. """
        
        lines= self.lines
        results = list(PerfTable().parse(lines))
        assert len(results) == self.num_test_ids # was all data read?
        assert len(dict.fromkeys([r.revision_id for r in results])) == (
                          1) # check for one unique revision id

        # check for self.num_test_ids unique test ids
        assert len(dict.fromkeys([r.test_id for r in results])) == ( 
                          self.num_test_ids) 

    def test_get_results(self):
        """check that get_results returns the right number of PerfResults"""

        perftable = PerfTable()
        perftable.add_lines(self.lines)
        results = list(perftable.get_results())
        assert len(results) == self.num_test_ids
        test_ids = perftable.list_values_of('test_id')
        # check that get_result returns all results for an empty test_id argument
        assert list(perftable.get_results()) == (
                          list(perftable.get_results(test_ids = test_ids)))
        # check that get_results returns only the requested (number of) results
        results = list(perftable.get_results(test_ids = test_ids[1:]))
        assert len(results) == self.num_test_ids -1

        
    def test_get_results_two_dates_same_revision_id(self):
        """check that PerfTable handles 2 dates in the data correctly"""

        # generate a second block of performance history data
        # with the same length as the first block
        new_date = float(self.lines[0].split(None, 2)[1]) + 1
        new_line = '%s %s %s' % (
            self.lines[0].split(None, 2)[0],
            new_date,
            self.lines[0].split(None, 2)[2],
        )
        lines = [new_line] + self.lines[1:] + self.lines

        perftable = PerfTable(lines)
        results = list(perftable.get_results())
        assert len(results) == 2* self.num_test_ids

        # get results for one (test_id, revision_id) pair
        # and check that get_result returns a result object for each date
        test_ids = perftable.list_values_of('test_id')[:1]
        revision_ids = perftable.list_values_of('revision_id')[:1]
        results = list(perftable.get_results(test_ids = test_ids,
                                             revision_ids = revision_ids))
        assert len(results) == 1+1
        r0 = results[0]
        r1 = results[1]
        assert r0 != r1
        #check that r0 and r1 are equal, except the date is different
        r0.date = r1.date = None
        assert r0.test_id == r1.test_id


    def test_list_values_of(self):
        perftable = PerfTable(self.lines)
        assert perftable.list_values_of('date') == [1152625530.0]
        assert perftable.list_values_of('revision_id') == ( 
                     ['hacker@canonical.com-20060705122759-0a3481a4647b16dc'])
        assert len(perftable.list_values_of('test_id')) == (
                          self.num_test_ids)
        # check number of unique test ids
        assert len(dict.fromkeys(
                                perftable.list_values_of('test_id')).keys()) == 2


class TestPerfResultCollection:
    """check the average and variance computation"""
    
    def test_property_elapsed_time(self):
        """check elapsed_time computated property"""
        p1 = PerfResult(elapsed_time=1, date=132.123)
        p2 = PerfResult(elapsed_time=2, date=142.1)
        sample = PerfResultCollection([p1,p2])
        assert sample.min_elapsed == 1

    
class TestPerfResultDelta:
    """check delta computations and the statistical tests"""
    
    def setup_method(self, meth):
        self.r1 = PerfResult(date=123123.123, elapsed_time = 10)
        self.r2 = PerfResult(date=123523.123, elapsed_time = 11)
        self.c1 = PerfResultCollection([self.r1])
        self.c2 = PerfResultCollection([self.r2])

    def test_attributes(self):
        """check delta computation"""
        delta = PerfResultDelta(self.c1, self.c2)
        assert delta.delta == ( 
                          self.c2.min_elapsed - self.c1.min_elapsed)
        assert round(delta.percent - 0.10, 7) == 0

    def test_delta_is_computed_from_one_value_only(self):
        delta = PerfResultDelta(self.c1, None)
        assert delta.delta == 0
        assert delta.percent == 0.0
        
        delta = PerfResultDelta(None, self.c2)
        assert delta.delta == 0
        assert delta.percent == 0.0

        
class TestPage:
    """check that every part of the page and the page itself can be
    generated without errors"""
    
    def setup_method(self, method):
        self.lines = testlines[:]
        self.perftable = PerfTable(self.lines)
            
    def check_serialize_html(self, html):
        """render unicode string from the given 'html' tree.

        :html: pyxml html object tree
        """
        x = unicode(html)

    def test_gen_image_map(self):
        """check image map for 20 Perf Results"""
        samples = [
            PerfResultCollection(
                [PerfResult(
                        elapsed_time=i,
                        test_id='test_id',
                        revision_id='revision %s' % (i,),
                        revision=i
                    )
                ]
            ) for i in range(20)
        ]
        x = Page().gen_image_map(samples, revisions=range(20))
        self.check_serialize_html(x)
    
    def test_report(self):
        """check revision report showing changes to prev revision"""
        
        p1 = [PerfResultCollection([PerfResult(elapsed_time=i,
                                           date=float(i),
                                           revision = 1,
                                           test_id='test123',
                                           revision_id='one')])
              for i in range(1, 4)]
        p2 = [PerfResultCollection([PerfResult(elapsed_time=i, 
                                           date=float(i), 
                                           test_id='test123',
                                           revision = 2,
                                           revision_id='two')])
              for i in range(2, 5)]

        deltas = [PerfResultDelta(*pair) for pair in zip(p1, p2)]
        self.check_serialize_html(Page().render_report(deltas))

    def test_header(self):
        """check header generation"""
        p1 = PerfResultCollection([PerfResult(revision_date=124.8, 
                                            nick="hello",
                                            revision=100)])
        p2 = PerfResultCollection([PerfResult(revision_date=12456.3, 
                                            nick="hello",
                                            revision=200 )])
        self.check_serialize_html(Page().render_header(
                p1.results[0], p2.results[0]))
        
    def test_table(self):
        """check main reporting table generation"""
        p1 = PerfResultCollection(
            [PerfResult(
                    elapsed_time=2, 
                    revision_date=124.8, 
                    revision=100, 
                    test_id=('bzrlib.benchmarks.bench_add.AddBenchmark.'
                                'test_one_add_kernel_like_tree'),
                )
            ]
        )
        p2 = PerfResultCollection(
            [PerfResult(
                    elapsed_time=3, 
                    revision_date=12456.3, 
                    revision=200,
                    test_id=('bzrlib.benchmarks.bench_add.AddBenchmark.'
                                'test_one_add_kernel_like_tree'),
                ),
            ]
        )

        samples = [
            PerfResultCollection(
                [PerfResult(
                        elapsed_time=i,
                        test_id='test_id',
                        revision_id='revision%s' % (i,),
                        revision = i
                    ),
                ],
            ) for i in range(20)
        ]

        images = [Page().gen_image_map(samples, revisions=range(20))
                    for i in range(4)]

        d1 = [PerfResultCollection(
                [PerfResult(
                        elapsed_time=i,
                        date = 130.0,
                        test_id='test123',
                        revision=1,
                        revision_id='one',
                    ),
                ],
            ) for i in range(1, 4)
        ]
        d2 = [PerfResultCollection(
                [PerfResult(
                        elapsed_time=i,
                        date = 140.0,
                        test_id='test123',
                        revision=2,
                        revision_id='two',
                    ),
                ],
            ) for i in range(2, 5)
        ]
        deltas = [PerfResultDelta(*pair) for pair in zip(d1, d2)]
        p = Page().render_table(deltas, images)
        self.check_serialize_html(p)

    def test_page_rendering_on_sample_dataset(self):
        perftable = PerfTable(testdata.splitlines())
        page = Page(perftable).render()
        self.check_serialize_html(page)

testdata = """

--date 1151350547.87  pqm@pqm.ubuntu.com-20060626193547-43661d1377f72b4d
 2744ms bzrlib.benchmarks.bench_add.AddBenchmark.test_one_add_kernel_like_tree

 3406ms bzrlib.benchmarks.bench_bench.MakeKernelLikeTreeBenchmark.test_make_kernel_like_tree

13492ms bzrlib.benchmarks.bench_checkout.CheckoutBenchmark.test_build_kernel_like_tree

27762ms bzrlib.benchmarks.bench_commit.CommitBenchmark.test_commit_kernel_like_tree

  155ms bzrlib.benchmarks.bench_inventory.InvBenchmark.test_make_10824_inv_entries

   58ms bzrlib.benchmarks.bench_log.LogBenchmark.test_cmd_log

  338ms bzrlib.benchmarks.bench_log.LogBenchmark.test_cmd_log_subprocess

  393ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log

   54ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_screenful

   37ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_screenful_line

   26ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_screenful_short

  687ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_verbose

   98ms bzrlib.benchmarks.bench_log.LogBenchmark.test_merge_log

  104ms bzrlib.benchmarks.bench_osutils.WalkDirsBenchmark.test_walkdirs_kernel_like_tree

  283ms bzrlib.benchmarks.bench_rocks.RocksBenchmark.test_rocks

 2222ms bzrlib.benchmarks.bench_status.StatusBenchmark.test_no_changes_known_kernel_like_tree

 1075ms bzrlib.benchmarks.bench_status.StatusBenchmark.test_no_ignored_unknown_kernel_like_tree

  510ms bzrlib.benchmarks.bench_transform.TransformBenchmark.test_canonicalize_path

  406ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_is_ignored_10824_calls

    3ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_is_ignored_single_call

  294ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_list_files_kernel_like_tree

 1081ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_list_files_unknown_kernel_like_tree

--date 1152797874.33  pqm@pqm.ubuntu.com-20060713133754-64c134fffd39fd99
 2588ms bzrlib.benchmarks.bench_add.AddBenchmark.test_one_add_kernel_like_tree

 3467ms bzrlib.benchmarks.bench_bench.MakeKernelLikeTreeBenchmark.test_make_kernel_like_tree

13637ms bzrlib.benchmarks.bench_checkout.CheckoutBenchmark.test_build_kernel_like_tree

28816ms bzrlib.benchmarks.bench_commit.CommitBenchmark.test_commit_kernel_like_tree

  207ms bzrlib.benchmarks.bench_inventory.InvBenchmark.test_make_10824_inv_entries

   54ms bzrlib.benchmarks.bench_log.LogBenchmark.test_cmd_log

  340ms bzrlib.benchmarks.bench_log.LogBenchmark.test_cmd_log_subprocess

  396ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log

   54ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_screenful

   41ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_screenful_line

   26ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_screenful_short

  691ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_verbose

   98ms bzrlib.benchmarks.bench_log.LogBenchmark.test_merge_log

  105ms bzrlib.benchmarks.bench_osutils.WalkDirsBenchmark.test_walkdirs_kernel_like_tree

  280ms bzrlib.benchmarks.bench_rocks.RocksBenchmark.test_rocks

 2229ms bzrlib.benchmarks.bench_status.StatusBenchmark.test_no_changes_known_kernel_like_tree

 1309ms bzrlib.benchmarks.bench_status.StatusBenchmark.test_no_ignored_unknown_kernel_like_tree

  504ms bzrlib.benchmarks.bench_transform.TransformBenchmark.test_canonicalize_path

   31ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_is_ignored_10824_calls

    0ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_is_ignored_single_call

  294ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_list_files_kernel_like_tree

  380ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_list_files_unknown_kernel_like_tree

--date 1152818329.6  pqm@pqm.ubuntu.com-20060713191849-c0cbdf94d208fa69
 1982ms bzrlib.benchmarks.bench_add.AddBenchmark.test_one_add_kernel_like_tree

 3748ms bzrlib.benchmarks.bench_bench.MakeKernelLikeTreeBenchmark.test_make_kernel_like_tree

13696ms bzrlib.benchmarks.bench_checkout.CheckoutBenchmark.test_build_kernel_like_tree

28351ms bzrlib.benchmarks.bench_commit.CommitBenchmark.test_commit_kernel_like_tree

  207ms bzrlib.benchmarks.bench_inventory.InvBenchmark.test_make_10824_inv_entries

   53ms bzrlib.benchmarks.bench_log.LogBenchmark.test_cmd_log

  334ms bzrlib.benchmarks.bench_log.LogBenchmark.test_cmd_log_subprocess

  397ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log

   55ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_screenful

   41ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_screenful_line

   26ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_screenful_short

  692ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_verbose

   99ms bzrlib.benchmarks.bench_log.LogBenchmark.test_merge_log

  107ms bzrlib.benchmarks.bench_osutils.WalkDirsBenchmark.test_walkdirs_kernel_like_tree

  281ms bzrlib.benchmarks.bench_rocks.RocksBenchmark.test_rocks

 2296ms bzrlib.benchmarks.bench_status.StatusBenchmark.test_no_changes_known_kernel_like_tree

 1119ms bzrlib.benchmarks.bench_status.StatusBenchmark.test_no_ignored_unknown_kernel_like_tree

  503ms bzrlib.benchmarks.bench_transform.TransformBenchmark.test_canonicalize_path

   30ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_is_ignored_10824_calls

    0ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_is_ignored_single_call

  507ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_list_files_kernel_like_tree

  353ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_list_files_unknown_kernel_like_tree

--date 1152977475.32  pqm@pqm.ubuntu.com-20060715153115-59f1601f31ecc38f
 1972ms bzrlib.benchmarks.bench_add.AddBenchmark.test_one_add_kernel_like_tree

 3512ms bzrlib.benchmarks.bench_bench.MakeKernelLikeTreeBenchmark.test_make_kernel_like_tree

13703ms bzrlib.benchmarks.bench_checkout.CheckoutBenchmark.test_build_kernel_like_tree

28276ms bzrlib.benchmarks.bench_commit.CommitBenchmark.test_commit_kernel_like_tree

  211ms bzrlib.benchmarks.bench_inventory.InvBenchmark.test_make_10824_inv_entries

   53ms bzrlib.benchmarks.bench_log.LogBenchmark.test_cmd_log

  333ms bzrlib.benchmarks.bench_log.LogBenchmark.test_cmd_log_subprocess

  397ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log

   56ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_screenful

   41ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_screenful_line

   26ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_screenful_short

  694ms bzrlib.benchmarks.bench_log.LogBenchmark.test_log_verbose

  100ms bzrlib.benchmarks.bench_log.LogBenchmark.test_merge_log

  110ms bzrlib.benchmarks.bench_osutils.WalkDirsBenchmark.test_walkdirs_kernel_like_tree

  271ms bzrlib.benchmarks.bench_rocks.RocksBenchmark.test_rocks

 2259ms bzrlib.benchmarks.bench_status.StatusBenchmark.test_no_changes_known_kernel_like_tree

 1548ms bzrlib.benchmarks.bench_status.StatusBenchmark.test_no_ignored_unknown_kernel_like_tree

  511ms bzrlib.benchmarks.bench_transform.TransformBenchmark.test_canonicalize_path

   30ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_is_ignored_10824_calls

    0ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_is_ignored_single_call

  296ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_list_files_kernel_like_tree

  387ms bzrlib.benchmarks.bench_workingtree.WorkingTreeBenchmark.test_list_files_unknown_kernel_like_tree
"""



from rpython.jit.backend.llsupport.test.ztranslation_test import TranslationRemoveTypePtrTest
from rpython.translator.translator import TranslationContext
from rpython.config.translationoption import DEFL_GC


class TestTranslationRemoveTypePtrPPC(TranslationRemoveTypePtrTest):
    def _get_TranslationContext(self):
        t = TranslationContext()
        t.config.translation.gc = DEFL_GC   # 'hybrid' or 'minimark'
        t.config.translation.list_comprehension_operations = True
        t.config.translation.gcremovetypeptr = True
        return t



import pytest
import platform
import sys
from os.path import commonprefix, dirname

THIS_DIR = dirname(__file__)

@pytest.hookimpl(tryfirst=True)
def pytest_ignore_collect(path, config):
    path = str(path)
    if sys.platform == 'win32' or platform.machine() == 's390x':
        if commonprefix([path, THIS_DIR]) == THIS_DIR:  # workaround for bug in pytest<3.0.5
            return True
